# Copyright 2015 OpenStack Foundation
# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from neutron_lib.db import model_base
from neutron_lib.db import standard_attr
import sqlalchemy as sa
from sqlalchemy import orm


class L2GatewayConnection(standard_attr.HasStandardAttributes,
                          model_base.BASEV2, model_base.HasProject,
                          model_base.HasId):
    """Define an l2 gateway connection between a l2 gateway and a network."""
    l2_gateway_id = sa.Column(sa.String(36),
                              sa.ForeignKey('l2gateways.id',
                                            ondelete='CASCADE'))
    network_id = sa.Column(sa.String(36),
                           sa.ForeignKey('networks.id', ondelete='CASCADE'),
                           nullable=False)
    segmentation_id = sa.Column(sa.Integer)
    __table_args__ = (sa.UniqueConstraint(l2_gateway_id,
                                          network_id),)
    api_collections = ["l2_gateway_connections"]


class L2GatewayInterface(model_base.BASEV2, model_base.HasId):
    """Define an l2 gateway interface."""
    interface_name = sa.Column(sa.String(255))
    device_id = sa.Column(sa.String(36),
                          sa.ForeignKey('l2gatewaydevices.id',
                                        ondelete='CASCADE'),
                          nullable=False)
    segmentation_id = sa.Column(sa.Integer)
    revises_on_change = ('devices', )


class L2GatewayDevice(standard_attr.HasStandardAttributes,
                      model_base.BASEV2, model_base.HasId):
    """Define an l2 gateway device."""
    device_name = sa.Column(sa.String(255), nullable=False)
    interfaces = orm.relationship(
        L2GatewayInterface,
        backref=sa.orm.backref("devices", uselist=False,
                               lazy='joined', load_on_pending=True),
        cascade='all,delete')
    l2_gateway_id = sa.Column(sa.String(36),
                              sa.ForeignKey('l2gateways.id',
                                            ondelete='CASCADE'),
                              nullable=False)
    revises_on_change = ('gateways', )
    api_collections = ["l2_gateway_devices"]


class L2Gateway(standard_attr.HasStandardAttributes,
                model_base.BASEV2, model_base.HasId, model_base.HasProject):
    """Define an l2 gateway."""
    name = sa.Column(sa.String(255))
    devices = orm.relationship(
        L2GatewayDevice,
        backref=sa.orm.backref("gateways", uselist=False,
                               lazy='joined', load_on_pending=True),
        cascade='all,delete')
    network_connections = orm.relationship(L2GatewayConnection,
                                           lazy='joined')
    api_collections = ["l2_gateways"]



# Copyright (c) 2014 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc


class API(object, metaclass=abc.ABCMeta):
    def __init__(self, context):
        self.context = context

    @abc.abstractmethod
    def transaction(self, check_error=False, log_errors=True, **kwargs):
        """Create a transaction

        :param check_error: Allow the transaction to raise an exception?
        :type check_error:  bool
        :param log_errors:  Log an error if the transaction fails?
        :type log_errors:   bool
        :returns: A new transaction
        :rtype: :class:`Transaction`
        """

    @abc.abstractmethod
    def db_find(self, table, *conditions, **kwargs):
        """Create a command to return find OVSDB records matching conditions

        :param table:     The OVS table to query
        :type table:      string
        :param conditions:The conditions to satisfy the query
        :type conditions: 3-tuples containing (column, operation, match)
                          Type of 'match' parameter MUST be identical to column
                          type
                          Examples:
                              atomic: ('tag', '=', 7)
                              map: ('external_ids' '=', {'iface-id': 'xxx'})
                              field exists?
                                  ('external_ids', '!=', {'iface-id', ''})
                              set contains?:
                                  ('protocols', '{>=}', 'OpenFlow13')
                          See the ovs-vsctl man page for more operations
        :param columns:   Limit results to only columns, None means all columns
        :type columns:    list of column names or None
        :returns:         :class:`Command` with [{'column', value}, ...] result
        """

    @abc.abstractmethod
    def get_physical_sw_list(self):
        """Create a command to list Physical Switches."""



# Copyright (c) 2015 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from unittest import mock

from neutron.db import agents_db
from neutron.tests.unit.plugins.ml2 import test_plugin
from neutron_lib import context as ctx
from neutron_lib import rpc as n_rpc

from networking_l2gw.db.l2gateway.ovsdb import lib as db
from networking_l2gw.services.l2gateway.common import l2gw_validators
from networking_l2gw.services.l2gateway.common import ovsdb_schema
from networking_l2gw.services.l2gateway import exceptions as l2gw_exc
from networking_l2gw.services.l2gateway.ovsdb import data
from networking_l2gw.services.l2gateway.service_drivers import agent_api
from networking_l2gw.services.l2gateway.service_drivers import rpc_l2gw

from oslo_utils import importutils


class TestL2gwRpcDriver(test_plugin.Ml2PluginV2TestCase):

    def setUp(self):
        super(TestL2gwRpcDriver, self).setUp()
        self.service_plugin = mock.MagicMock()
        load_driver = mock.MagicMock()
        self.service_plugin._load_drivers.return_value = load_driver
        self.service_plugin._get_driver_for_provider.return_value = load_driver
        self.plugin = rpc_l2gw.L2gwRpcDriver(self.service_plugin)
        self.plugin.agent_rpc = mock.MagicMock()
        self.ovsdb_identifier = 'fake_ovsdb_id'
        self.context = mock.ANY

    @mock.patch.object(importutils,
                       'import_object')
    @mock.patch.object(agents_db,
                       'AgentExtRpcCallback')
    @mock.patch.object(n_rpc,
                       'Connection')
    @mock.patch.object(n_rpc.Connection,
                       'consume_in_threads')
    @mock.patch.object(rpc_l2gw.LOG,
                       'debug')
    @mock.patch.object(rpc_l2gw.L2gwRpcDriver,
                       'start_l2gateway_agent_scheduler')
    def test_l2rpcdriver_init(self, scheduler, debug, consume_in_thread,
                              create_conn, agent_calback, import_obj):
        rpc_l2gw.L2gwRpcDriver(self.service_plugin)
        rpc_conn = create_conn.return_value
        rpc_conn.create_consumer.assert_called_with(
            mock.ANY, [mock.ANY, mock.ANY], fanout=mock.ANY)
        rpc_conn.consume_in_threads.assert_called_with()
        self.assertTrue(import_obj.called)
        self.assertTrue(agent_calback.called)
        self.assertTrue(create_conn.called)
        self.assertTrue(debug.called)
        self.assertTrue(scheduler.called)

    def test_validate_connection(self):
        fake_connection = {'l2_gateway_id': 'fake_l2gw_id',
                           'network_id': 'fake_network_id',
                           'segmentation_id': 100}
        fake_net_seg_list = [{'id': 'fake_id',
                              'network_type': 'fake_vxlan',
                              'physical_network': 'fake_phy_net',
                              'segmentation_id': 100}]
        fake_l2_gw = {'id': 'fake_l2gw_id'}
        fake_tenant_id = 'fake_tenant_id'
        fake_filters = {'network_id': ['fake_network_id'],
                        'tenant_id': [fake_tenant_id],
                        'l2_gateway_id': ['fake_l2gw_id']}
        with mock.patch.object(self.service_plugin,
                               '_is_vlan_configured_on_any_interface_for_l2gw',
                               return_value=False) as is_vlan, \
            mock.patch.object(l2gw_validators,
                              'validate_network_mapping_list',
                              return_value='fake_network_id') as val_ntwk, \
            mock.patch.object(self.service_plugin,
                              '_get_network_segments',
                              return_value=fake_net_seg_list), \
            mock.patch.object(self.service_plugin,
                              '_get_network',
                              return_value=True) as get_network, \
            mock.patch.object(self.service_plugin,
                              '_get_l2_gateway',
                              return_value=fake_l2_gw) as get_l2gw, \
            mock.patch.object(self.service_plugin,
                              '_retrieve_gateway_connections',
                              return_value=False) as ret_gw_conn, \
            mock.patch.object(self.service_plugin,
                              '_get_tenant_id_for_create',
                              return_value=fake_tenant_id) as get_ten_id, \
            mock.patch.object(self.service_plugin,
                              'get_l2_gateway_connections',
                              return_value=False) as get_l2_gw_conn, \
            mock.patch.object(
                self.plugin,
                '_check_port_fault_status_and_switch_fault_status') as pf_sf:
            self.plugin._validate_connection(self.context, fake_connection)
            is_vlan.assert_called_with(self.context, 'fake_l2gw_id')
            val_ntwk.assert_called_with(fake_connection, False)
            get_network.assert_called_with(self.context, 'fake_network_id')
            get_l2gw.assert_called_with(self.context, 'fake_l2gw_id')
            pf_sf.assert_called_with(self.context, 'fake_l2gw_id')
            ret_gw_conn.assert_called_with(self.context,
                                           'fake_l2gw_id',
                                           fake_connection)
            get_ten_id.assert_called_with(self.context, fake_l2_gw)
            get_l2_gw_conn.assert_called_with(self.context,
                                              filters=fake_filters)

    @mock.patch.object(db,
                       'get_physical_switch_by_name')
    @mock.patch.object(db,
                       'get_logical_switch_by_name')
    @mock.patch.object(db,
                       'get_physical_port_by_name_and_ps')
    def test_process_port_list(self, get_pp, get_ls, get_ps):
        fake_connection = {'l2_gateway_id': 'fake_l2gw_id',
                           'network_id': 'fake_network_id',
                           'segmentation_id': 100}
        fake_device = {'id': 'fake_device_id',
                       'device_name': 'fake_device_name'}
        fake_method = 'CREATE'
        fake_interface = {'interface_name': 'fake_interface_name'}
        fake_interface_list = [fake_interface]
        fake_physical_switch = {'uuid': 'fake_uuid',
                                'ovsdb_identifier': 'fake_ovsdb_id'}
        fake_pp_dict = {'interface_name': 'fake_interface_name',
                        'ovsdb_identifier': 'fake_ovsdb_id',
                        'physical_switch_id': 'fake_uuid',
                        'logical_switch_name': 'fake_network_id',
                        'uuid': 'fake_uuid',
                        'name': 'fake_name'}
        fake_logical_switch = {'uuid': 'fake_uuid',
                               'name': 'fake_network_id'}
        fake_physical_port = {'uuid': 'fake_uuid',
                              'name': 'fake_name'}
        get_ps.return_value = fake_physical_switch
        get_ls.return_value = fake_logical_switch
        get_pp.return_value = fake_physical_port
        with mock.patch.object(
                self.service_plugin,
                'get_l2gateway_interfaces_by_device_id',
                return_value=fake_interface_list) as get_intf, \
            mock.patch.object(self.plugin,
                              '_generate_port_list') as gen_port_list:
            self.plugin._process_port_list(self.context, fake_device,
                                           fake_connection,
                                           fake_method)
            get_intf.assert_called_with(self.context, 'fake_device_id')
            get_ps.assert_called_with(self.context, 'fake_device_name')
            get_pp.assert_called_with(self.context, fake_pp_dict)
            get_ls.assert_called_with(self.context, fake_pp_dict)
            gen_port_list.assert_called_with(
                self.context, fake_method, 100, fake_interface,
                fake_pp_dict, 'fake_uuid', fake_connection)

    def test_generate_port_list_for_create(self):
        fake_method = 'CREATE'
        fake_interface = {'interface_name': 'fake_interface_name'}
        fake_pp_dict = {'interface_name': 'fake_interface_name',
                        'ovsdb_identifier': 'fake_ovsdb_id',
                        'physical_switch_id': 'fake_uuid',
                        'logical_switch_name': 'fake_network_id',
                        'uuid': 'fake_uuid',
                        'name': 'fake_name'}
        fake_vlan_binding = {'vlan': 100,
                             'logical_switch_uuid': 'fake_uuid'}
        fake_vlan_binding_list = [fake_vlan_binding]
        with mock.patch.object(
            db, 'get_all_vlan_bindings_by_physical_port',
            return_value=fake_vlan_binding_list) as (
                get_vlan):
            self.plugin._generate_port_list(
                self.context, fake_method, 101, fake_interface,
                fake_pp_dict, 'fake_uuid')
            get_vlan.assert_called_with(self.context, fake_pp_dict)

    def test_generate_port_list_for_create_for_duplicate_seg_id(self):
        fake_method = 'CREATE'
        fake_interface = {'interface_name': 'fake_interface_name'}
        fake_pp_dict = {'interface_name': 'fake_interface_name',
                        'ovsdb_identifier': 'fake_ovsdb_id',
                        'physical_switch_id': 'fake_uuid',
                        'logical_switch_name': 'fake_network_id',
                        'uuid': 'fake_uuid',
                        'name': 'fake_name'}
        fake_vlan_binding = {'vlan': 100,
                             'logical_switch_uuid': 'fake_uuid'}
        fake_vlan_binding_list = [fake_vlan_binding]
        with mock.patch.object(
            db, 'get_all_vlan_bindings_by_physical_port',
            return_value=fake_vlan_binding_list) as (
                get_vlan):
            self.assertRaises(l2gw_exc.L2GatewayDuplicateSegmentationID,
                              self.plugin._generate_port_list,
                              self.context, fake_method, 100,
                              fake_interface, fake_pp_dict, 'fake_uuid')
            get_vlan.assert_called_with(self.context, fake_pp_dict)

    def test_generate_port_list_for_delete(self):
        fake_connection = {'l2_gateway_id': 'fake_l2gw_id',
                           'network_id': 'fake_network_id',
                           'segmentation_id': 100}
        fake_method = 'DELETE'
        fake_interface = {'interface_name': 'fake_interface_name'}
        fake_pp_dict = {'interface_name': 'fake_interface_name',
                        'ovsdb_identifier': 'fake_ovsdb_id',
                        'physical_switch_id': 'fake_uuid',
                        'logical_switch_name': 'fake_network_id',
                        'uuid': 'fake_uuid'}
        fake_vlan_binding = {'vlan': 100,
                             'logical_switch_uuid': 'fake_uuid'}
        fake_vlan_binding_list = [fake_vlan_binding]

        vlan_dict = {'vlan': 100,
                     'logical_switch_uuid': 'fake_uuid'}
        physical_port = ovsdb_schema.PhysicalPort(
            uuid='fake_uuid',
            name='fake_interface_name',
            phys_switch_id='fake_uuid',
            vlan_binding_dicts=None,
            port_fault_status=None)
        phys_port_dict = physical_port.__dict__
        phys_port_dict['vlan_bindings'] = [vlan_dict]
        with mock.patch.object(
            db,
            'get_all_vlan_bindings_by_physical_port',
            return_value=fake_vlan_binding_list) as (
                get_vlan):
            port = self.plugin._generate_port_list(
                self.context, fake_method, 100, fake_interface,
                fake_pp_dict, 'fake_uuid', fake_connection)
            get_vlan.assert_called_with(self.context, fake_pp_dict)
            self.assertEqual(phys_port_dict, port)

    def test_get_ip_details(self):
        fake_port = {'binding:host_id': 'fake_host',
                     'fixed_ips': [{'ip_address': 'fake_ip'}]}
        fake_agent = {'configurations': {'tunneling_ip': 'fake_tun_ip'}}
        with mock.patch.object(self.plugin,
                               '_get_agent_details',
                               return_value=fake_agent) as get_agent:
            (ret_dst_ip, ret_ip_add) = self.plugin._get_ip_details(
                self.context, fake_port)
            get_agent.assert_called_with(self.context, 'fake_host')
            self.assertEqual('fake_tun_ip', ret_dst_ip)
            self.assertEqual('fake_ip', ret_ip_add)

    def test_get_agent_details_for_no_ovs_agent(self):
        core_plugin = mock.PropertyMock()
        type(self.service_plugin)._core_plugin = core_plugin
        (self.service_plugin._core_plugin.get_agents.
         return_value) = []
        self.assertRaises(l2gw_exc.L2AgentNotFoundByHost,
                          self.plugin._get_agent_details,
                          self.context, 'fake_host')

    def test_get_network_details(self):
        fake_network = {'id': 'fake_network_id',
                        'name': 'fake_network_name',
                        'provider:segmentation_id': 'fake_key'}
        core_plugin = mock.PropertyMock()
        type(self.service_plugin)._core_plugin = core_plugin
        (self.service_plugin._core_plugin.get_network.
         return_value) = fake_network

    def test_get_port_details(self):
        fake_port = {'binding:host_id': 'fake_host',
                     'fixed_ips': [{'ip_address': 'fake_ip'}],
                     'mac_address': 'fake_mac_add'}
        fake_port_list = [fake_port]
        core_plugin = mock.PropertyMock()
        type(self.service_plugin)._core_plugin = core_plugin
        (self.service_plugin._core_plugin.get_ports.
         return_value) = fake_port_list

    def test_get_agent_details(self):
        fake_agent = [{'configurations': {'tunneling_ip': 'fake_tun_ip'}}]
        core_plugin = mock.PropertyMock()
        type(self.service_plugin)._core_plugin = core_plugin
        (self.service_plugin._core_plugin.get_agents.return_value) = fake_agent

    def test_get_logical_switch_dict(self):
        fake_logical_switch = {'uuid': 'fake_uuid',
                               'name': 'fake_network_id'}
        fake_ls = None
        fake_connection = {'l2_gateway_id': 'fake_l2gw_id',
                           'network_id': 'fake_network_id',
                           'segmentation_id': 100}
        fake_network = {'id': 'fake_network_id',
                        'name': 'fake_network_name',
                        'provider:network_type': 'vxlan',
                        'provider:segmentation_id': 'fake_key'}
        fake_ls_dict = {'uuid': 'fake_uuid',
                        'name': 'fake_network_id',
                        'description': 'fake_network_name',
                        'key': 'fake_key'}
        fake_ls_dict_without_ls = {'uuid': None,
                                   'name': 'fake_network_id',
                                   'description': 'fake_network_name',
                                   'key': 'fake_key'}
        with mock.patch.object(self.plugin,
                               '_get_network_details',
                               return_value=fake_network) as get_network:
            ret_ls_dict = self.plugin._get_logical_switch_dict(
                self.context, fake_logical_switch, fake_connection)
            ret_ls_dict_without_ls = self.plugin._get_logical_switch_dict(
                self.context, fake_ls, fake_connection)
            get_network.assert_called_with(self.context, 'fake_network_id')
            self.assertEqual(fake_ls_dict, ret_ls_dict)
            self.assertEqual(fake_ls_dict_without_ls, ret_ls_dict_without_ls)

    def test_get_logical_switch_dict_for_multi_segment_network(self):
        fake_logical_switch = {'uuid': 'fake_uuid',
                               'name': 'fake_network_id'}
        fake_ls = None
        fake_connection = {'l2_gateway_id': 'fake_l2gw_id',
                           'network_id': 'fake_network_id',
                           'segmentation_id': 100}
        fake_network = {'id': 'fake_network_id',
                        'name': 'fake_network_name',
                        'segments': [{"provider:network_type": "vxlan",
                                      "provider:segmentation_id": 'fake_key'},
                                     {"provider:network_type": "vlan",
                                      "provider:segmentation_id": 'fake_key'}]}
        fake_ls_dict = {'uuid': 'fake_uuid',
                        'name': 'fake_network_id',
                        'description': 'fake_network_name',
                        'key': 'fake_key'}
        fake_ls_dict_without_ls = {'uuid': None,
                                   'name': 'fake_network_id',
                                   'description': 'fake_network_name',
                                   'key': 'fake_key'}
        with mock.patch.object(self.plugin,
                               '_get_network_details',
                               return_value=fake_network) as get_network:
            ret_ls_dict = self.plugin._get_logical_switch_dict(
                self.context, fake_logical_switch, fake_connection)
            ret_ls_dict_without_ls = self.plugin._get_logical_switch_dict(
                self.context, fake_ls, fake_connection)
            get_network.assert_called_with(self.context, 'fake_network_id')
            self.assertEqual(fake_ls_dict, ret_ls_dict)
            self.assertEqual(fake_ls_dict_without_ls, ret_ls_dict_without_ls)

    def test_get_logical_switch_dict_for_non_Vxlan_networks(self):
        fake_logical_switch = {'uuid': 'fake_uuid',
                               'name': 'fake_network_id'}
        fake_connection = {'l2_gateway_id': 'fake_l2gw_id',
                           'network_id': 'fake_network_id',
                           'segmentation_id': 100}
        fake_network = {'id': 'fake_network_id',
                        'name': 'fake_network_name',
                        'segments': [{"provider:network_type": "vlan",
                                      "provider:segmentation_id": 'fake_key'},
                                     {"provider:network_type": "gre",
                                      "provider:segmentation_id": 'fake_key'}]}
        with mock.patch.object(self.plugin,
                               '_get_network_details',
                               return_value=fake_network) as get_network:
            self.assertRaises(l2gw_exc.VxlanSegmentationIDNotFound,
                              self.plugin._get_logical_switch_dict,
                              self.context, fake_logical_switch,
                              fake_connection)
            get_network.assert_called_with(self.context, 'fake_network_id')

    def test_get_logical_switch_dict_for_multiple_vxlan_segments(self):
        fake_logical_switch = {'uuid': 'fake_uuid',
                               'name': 'fake_network_id'}
        fake_connection = {'l2_gateway_id': 'fake_l2gw_id',
                           'network_id': 'fake_network_id',
                           'segmentation_id': 100}
        fake_network = {'id': 'fake_network_id',
                        'name': 'fake_network_name',
                        'segments': [{"provider:network_type": "vxlan",
                                      "provider:segmentation_id": 'seg_1'},
                                     {"provider:network_type": "vxlan",
                                      "provider:segmentation_id": 'seg_2'}]}
        with mock.patch.object(self.plugin,
                               '_get_network_details',
                               return_value=fake_network) as get_network:
            self.assertRaises(l2gw_exc.MultipleVxlanSegmentsFound,
                              self.plugin._get_logical_switch_dict,
                              self.context, fake_logical_switch,
                              fake_connection)
            get_network.assert_called_with(self.context, 'fake_network_id')

    @mock.patch.object(db,
                       'get_physical_locator_by_dst_ip')
    def test_get_locator_list(self, get_pl_by_dst_ip):
        fake_dst_ip = 'fake_tun_ip'
        fake_ovsdb_id = 'fake_ovsdb_id'
        fake_mac_list = []
        fake_locator_list = []
        fake_locator_dict = {'uuid': 'fake_uuid',
                             'dst_ip': 'fake_tun_ip',
                             'ovsdb_identifier': 'fake_ovsdb_id',
                             'macs': []}
        fake_pl = {'uuid': 'fake_uuid',
                   'dst_ip': 'fake_tun_ip'}
        fale_pl_list = [fake_locator_dict]
        get_pl_by_dst_ip.return_value = fake_pl
        with mock.patch.object(
                self.plugin,
                '_get_physical_locator_dict',
                return_value=fake_locator_dict) as get_pl_dict:
            ret_pl_list = self.plugin._get_locator_list(
                self.context, fake_dst_ip, fake_ovsdb_id,
                fake_mac_list, fake_locator_list)
            get_pl_by_dst_ip.assert_called_with(self.context,
                                                fake_locator_dict)
            get_pl_dict.assert_called_with(fake_dst_ip,
                                           'fake_uuid', fake_mac_list)
            self.assertEqual(fale_pl_list, ret_pl_list)

    @mock.patch.object(db,
                       'get_physical_switch_by_name')
    def test_get_identifer_list(self, get_ps):
        fake_connection = {'l2_gateway_id': 'fake_l2gw_id',
                           'network_id': 'fake_network_id',
                           'segmentation_id': 100}
        fake_device = {'id': 'fake_device_id',
                       'device_name': 'fake_device_name'}
        fake_device_list = [fake_device]
        fake_physical_switch = {'uuid': 'fake_uuid',
                                'ovsdb_identifier': 'fake_ovsdb_id'}
        fake_identifier_set = set(['fake_ovsdb_id'])
        get_ps.return_value = fake_physical_switch
        with mock.patch.object(self.service_plugin,
                               'get_l2gateway_devices_by_gateway_id',
                               return_value=fake_device_list):
            ret_value = self.plugin._get_identifer_list(self.context,
                                                        fake_connection)
            (self.service_plugin.get_l2gateway_devices_by_gateway_id.
             assert_called_with(mock.ANY, mock.ANY))
            db.get_physical_switch_by_name.assert_called_with(
                self.context, 'fake_device_name')
            self.assertEqual(fake_identifier_set, ret_value)

    def test_get_set_of_ovsdb_ids(self):
        fake_connection = {'id': 'fake_id',
                           'l2_gateway_id': 'fake_l2gw_id',
                           'network_id': 'fake_network_id',
                           'segmentation_id': 100}
        fake_gw_conn_ovsdb_set = set(['fake_ovsdb_id'])
        fake_connection_list = [fake_connection]
        with mock.patch.object(self.service_plugin,
                               'get_l2_gateway_connections',
                               return_value=fake_connection_list):
            ret_value = self.plugin._get_set_of_ovsdb_ids(
                self.context, fake_connection,
                fake_gw_conn_ovsdb_set)
            self.service_plugin.get_l2_gateway_connections.assert_called_with(
                mock.ANY, filters=mock.ANY)
            self.assertEqual(fake_gw_conn_ovsdb_set, ret_value)

    def test_remove_vm_macs(self):
        fake_network_id = 'fake_network_id'
        fake_ovsdb_id_set = set(['fake_ovsdb_id'])
        fake_port = {'binding:host_id': 'fake_host',
                     'fixed_ips': [{'ip_address': 'fake_ip'}],
                     'mac_address': 'fake_mac_add'}
        fake_port_list = [fake_port]
        with mock.patch.object(self.plugin,
                               '_get_port_details',
                               return_value=fake_port_list) as get_port, \
            mock.patch.object(self.plugin,
                              'delete_port_mac') as delete_mac:
            self.plugin._remove_vm_macs(self.context,
                                        fake_network_id,
                                        fake_ovsdb_id_set)
            get_port.assert_called_with(self.context, fake_network_id)
            delete_mac.assert_called_with(self.context,
                                          fake_port_list)

    def test_add_port_mac(self):
        fake_ip1 = "fake_ip1"
        fake_ip2 = "fake_ip2"
        fake_network_dict = {'provider:segmentation_id': 100,
                             'name': 'fake_name',
                             'id': 'fake_network_id'}
        fake_conn_dict = {'id': 'fake_conn_id',
                          'l2_gateway_id': 'fake_gateway_id'}
        fake_conn_list = [fake_conn_dict]
        fake_logical_switch = {'ovsdb_identifier': 'fake_ovsdb_id',
                               'uuid': 'fake_network_id',
                               'description': 'fake_name'}
        fake_logical_switch_list = [fake_logical_switch]
        fake_locator_dict = {'ovsdb_identifier': 'fake_ovsdb_id',
                             'uuid': 'fake_locator_id',
                             'dst_ip': 'fake_ip2'}
        fake_pl_dict = {'ovsdb_identifier': 'fake_ovsdb_id',
                        'dst_ip': 'fake_ip1'}
        fake_mac_dict = {'mac': 'fake_mac',
                         'logical_switch_id': 'fake_network_id',
                         'physical_locator_id': 'fake_locator_id',
                         'ip_address': 'fake_ip1'}
        fake_dict = fake_mac_dict
        fake_dict['ovsdb_identifier'] = 'fake_ovsdb_id'
        core_plugin = mock.PropertyMock()
        type(self.service_plugin)._core_plugin = core_plugin
        (self.service_plugin._core_plugin.get_port.
         return_value) = {'device_owner': 'fake_owner',
                          'network_id': 'fake_network_id',
                          'mac_address': 'fake_mac'}
        ovsdb_identifier = 'fake_ovsdb_id'
        with mock.patch.object(self.plugin,
                               '_get_ip_details',
                               return_value=(fake_ip1,
                                             fake_ip2)), \
            mock.patch.object(self.plugin,
                              '_get_network_details',
                              return_value=fake_network_dict) as get_network, \
            mock.patch.object(self.service_plugin,
                              'get_l2_gateway_connections',
                              return_value=fake_conn_list) as get_l2gw_conn, \
            mock.patch.object(self.plugin,
                              '_form_physical_locator_schema',
                              return_value=fake_locator_dict) as get_pl, \
            mock.patch.object(ovsdb_schema,
                              'UcastMacsRemote') as mock_ucmr, \
            mock.patch.object(self.plugin,
                              '_get_dict',
                              return_value=fake_dict) as get_dict, \
            mock.patch.object(db,
                              'get_ucast_mac_remote_by_mac_and_ls',
                              return_value=False) as get_ucast_mac, \
            mock.patch.object(self.plugin.agent_rpc,
                              'add_vif_to_gateway') as add_rpc, \
            mock.patch.object(self.plugin.agent_rpc,
                              'update_vif_to_gateway') as update_rpc, \
            mock.patch.object(db,
                              'get_all_logical_switches_by_name',
                              return_value=fake_logical_switch_list):
            remote_mac = ovsdb_schema.UcastMacsRemote(
                None, fake_dict['mac'], fake_logical_switch['uuid'],
                fake_locator_dict['uuid'],
                fake_ip2)
            mock_ucmr.return_value = remote_mac
            self.plugin.add_port_mac(self.context, fake_dict)
            get_network.assert_called_with(self.context, 'fake_network_id')
            get_l2gw_conn.assert_called_with(
                self.context, filters={'network_id': ['fake_network_id']})
            get_pl.assert_called_with(self.context, fake_pl_dict)
            get_ucast_mac.assert_called_with(self.context, fake_dict)
            get_dict.assert_called_with(remote_mac)
            mock_ucmr.assert_called_with(
                uuid=None,
                mac=fake_dict['mac'],
                logical_switch_id=fake_logical_switch['uuid'],
                physical_locator_id=fake_locator_dict['uuid'],
                ip_address=fake_ip2)
            add_rpc.assert_called_with(
                self.context, ovsdb_identifier,
                fake_logical_switch, fake_locator_dict, fake_dict)
            self.assertFalse(update_rpc.called)

    @mock.patch.object(db,
                       'get_all_logical_switches_by_name')
    @mock.patch.object(db,
                       'get_ucast_mac_remote_by_mac_and_ls',
                       return_value=True)
    @mock.patch.object(agent_api.L2gatewayAgentApi,
                       'delete_vif_from_gateway')
    @mock.patch.object(db,
                       'get_logical_switch_by_name')
    def test_delete_port_mac_for_multiple_vlan_bindings(
            self, get_ls, delete_rpc, get_mac, get_all_ls):
        fake_port_list = [{'network_id': 'fake_network_id',
                           'device_owner': 'fake_owner',
                           'mac_address': 'fake_mac',
                           'ovsdb_identifier': 'fake_ovsdb_id'}]
        fake_logical_switch_dict = {'uuid': 'fake_uuid',
                                    'ovsdb_identifier': 'fake_ovsdb_id'}
        fake_logical_switch_list = [fake_logical_switch_dict]
        lg_dict = {'logical_switch_name': 'fake_network_id',
                   'ovsdb_identifier': 'fake_ovsdb_id'}
        get_all_ls.return_value = fake_logical_switch_list
        get_ls.return_value = fake_logical_switch_dict
        with mock.patch.object(self.service_plugin,
                               'get_l2_gateway_connections',
                               return_value=[1, 2]):
            self.plugin.delete_port_mac(self.context, fake_port_list)
            self.assertFalse(get_all_ls.called)
            get_ls.assert_called_with(self.context, lg_dict)
            self.assertFalse(get_mac.called)
            self.assertFalse(delete_rpc.called)

    def test_add_port_mac_with_ovsdb_server_down(self):
        "Test case to test add_port_mac when the OVSDB server is down."
        fake_ip1 = "fake_ip1"
        fake_ip2 = "fake_ip2"
        fake_network_dict = {'provider:segmentation_id': 100,
                             'name': 'fake_name',
                             'id': 'fake_network_id'}
        fake_conn_dict = {'id': 'fake_conn_id',
                          'l2_gateway_id': 'fake_gateway_id'}
        fake_conn_list = [fake_conn_dict]
        fake_logical_switch = {'ovsdb_identifier': 'fake_ovsdb_id',
                               'uuid': 'fake_network_id',
                               'description': 'fake_name'}
        fake_logical_switch_list = [fake_logical_switch]
        fake_locator_dict = {'ovsdb_identifier': 'fake_ovsdb_id',
                             'uuid': 'fake_locator_id',
                             'dst_ip': 'fake_ip2'}
        fake_pl_dict = {'ovsdb_identifier': 'fake_ovsdb_id',
                        'dst_ip': 'fake_ip1'}
        fake_mac_dict = {'mac': 'fake_mac',
                         'logical_switch_id': 'fake_network_id',
                         'physical_locator_id': 'fake_locator_id',
                         'ip_address': 'fake_ip1'}
        fake_dict = fake_mac_dict
        fake_dict['ovsdb_identifier'] = 'fake_ovsdb_id'
        core_plugin = mock.PropertyMock()
        type(self.service_plugin)._core_plugin = core_plugin
        (self.service_plugin._core_plugin.get_port.
         return_value) = {'device_owner': 'fake_owner',
                          'network_id': 'fake_network_id',
                          'mac_address': 'fake_mac'}
        ovsdb_identifier = 'fake_ovsdb_id'
        with mock.patch.object(self.plugin,
                               '_get_ip_details',
                               return_value=(fake_ip1,
                                             fake_ip2)), \
            mock.patch.object(self.plugin,
                              '_get_network_details',
                              return_value=fake_network_dict) as get_network, \
            mock.patch.object(self.service_plugin,
                              'get_l2_gateway_connections',
                              return_value=fake_conn_list), \
            mock.patch.object(self.plugin,
                              '_form_physical_locator_schema',
                              return_value=fake_locator_dict) as get_pl, \
            mock.patch.object(ovsdb_schema,
                              'UcastMacsRemote') as mock_ucmr, \
            mock.patch.object(self.plugin,
                              '_get_dict',
                              return_value=fake_dict) as get_dict, \
            mock.patch.object(db,
                              'get_ucast_mac_remote_by_mac_and_ls',
                              return_value=False) as get_ucast_mac, \
            mock.patch.object(self.plugin.agent_rpc,
                              'add_vif_to_gateway',
                              side_effect=RuntimeError) as add_rpc, \
            mock.patch.object(self.plugin.agent_rpc,
                              'update_vif_to_gateway') as update_rpc, \
            mock.patch.object(
                db,
                'add_pending_ucast_mac_remote') as add_pending_mac, \
            mock.patch.object(db, 'get_all_logical_switches_by_name',
                              return_value=fake_logical_switch_list):
            remote_mac = ovsdb_schema.UcastMacsRemote(
                None, fake_dict['mac'], fake_logical_switch['uuid'],
                fake_locator_dict['uuid'],
                fake_ip2)
            mock_ucmr.return_value = remote_mac
            self.plugin.add_port_mac(self.context, fake_dict)
            get_network.assert_called_with(mock.ANY, mock.ANY)
            self.service_plugin.get_l2_gateway_connections.assert_called_with(
                mock.ANY, filters=mock.ANY)
            get_pl.assert_called_with(self.context, fake_pl_dict)
            get_dict.assert_called_with(remote_mac)
            get_ucast_mac.assert_called_with(self.context, fake_dict)
            add_rpc.assert_called_with(
                self.context, ovsdb_identifier,
                fake_logical_switch, fake_locator_dict, fake_dict)
            self.assertFalse(update_rpc.called)
            self.assertTrue(add_pending_mac.called)

    def test_add_port_mac_vm_migrate(self):
        fake_ip1 = "fake_ip1"
        fake_ip2 = "fake_ip2"
        fake_network_dict = {'provider:segmentation_id': 100,
                             'name': 'fake_name',
                             'id': 'fake_network_id'}
        fake_conn_dict = {'id': 'fake_conn_id',
                          'l2_gateway_id': 'fake_gateway_id'}
        fake_conn_list = [fake_conn_dict]
        fake_logical_switch = {'ovsdb_identifier': 'fake_ovsdb_id',
                               'uuid': 'fake_network_id',
                               'description': 'fake_name'}
        fake_logical_switch_list = [fake_logical_switch]
        fake_locator_dict = {'ovsdb_identifier': 'fake_ovsdb_id',
                             'uuid': 'fake_locator_id',
                             'dst_ip': 'fake_ip2'}
        fake_pl_dict = {'ovsdb_identifier': 'fake_ovsdb_id',
                        'dst_ip': 'fake_ip1'}
        fake_mac_dict = {'mac': 'fake_mac',
                         'uuid': 'fake_mac_id',
                         'locator': 'fake_locator_id2',
                         'logical_switch_id': 'fake_network_id',
                         'physical_locator_id': 'fake_locator_id1',
                         'ip_address': 'fake_ip1'}
        fake_dict = fake_mac_dict
        fake_dict['ovsdb_identifier'] = 'fake_ovsdb_id'
        core_plugin = mock.PropertyMock()
        type(self.service_plugin)._core_plugin = core_plugin
        (self.service_plugin._core_plugin.get_port.
         return_value) = {'device_owner': 'fake_owner',
                          'network_id': 'fake_network_id',
                          'mac_address': 'fake_mac'}
        ovsdb_identifier = 'fake_ovsdb_id'
        with mock.patch.object(self.plugin,
                               '_get_ip_details',
                               return_value=(fake_ip1,
                                             fake_ip2)), \
            mock.patch.object(self.plugin,
                              '_get_network_details',
                              return_value=fake_network_dict) as get_network, \
            mock.patch.object(self.service_plugin,
                              'get_l2_gateway_connections',
                              return_value=fake_conn_list) as get_l2gw_conn, \
            mock.patch.object(self.plugin,
                              '_form_physical_locator_schema',
                              return_value=fake_locator_dict) as get_pl, \
            mock.patch.object(self.plugin,
                              '_get_dict',
                              return_value=fake_dict), \
            mock.patch.object(db,
                              'get_ucast_mac_remote_by_mac_and_ls',
                              return_value=fake_mac_dict) as get_ucast_mac, \
            mock.patch.object(self.plugin.agent_rpc,
                              'add_vif_to_gateway') as add_rpc, \
            mock.patch.object(self.plugin.agent_rpc,
                              'update_vif_to_gateway') as update_rpc, \
            mock.patch.object(db,
                              'get_all_logical_switches_by_name',
                              return_value=fake_logical_switch_list):
            self.plugin.add_port_mac(self.context, fake_dict)
            get_network.assert_called_with(self.context, 'fake_network_id')
            get_l2gw_conn.assert_called_with(
                self.context, filters={'network_id': ['fake_network_id']})
            get_pl.assert_called_with(self.context, fake_pl_dict)
            get_ucast_mac.assert_called_with(self.context, fake_dict)
            self.assertFalse(add_rpc.called)
            update_rpc.assert_called_with(
                self.context, ovsdb_identifier,
                fake_locator_dict, fake_dict)

    def test_add_port_mac_vm_migrate_with_ovsdb_server_down(self):
        "Test case to test update_port_mac when the OVSDB server is down."
        fake_ip1 = "fake_ip1"
        fake_ip2 = "fake_ip2"
        fake_network_dict = {'provider:segmentation_id': 100,
                             'name': 'fake_name',
                             'id': 'fake_network_id'}
        fake_conn_dict = {'id': 'fake_conn_id',
                          'l2_gateway_id': 'fake_gateway_id'}
        fake_conn_list = [fake_conn_dict]
        fake_logical_switch = {'ovsdb_identifier': 'fake_ovsdb_id',
                               'uuid': 'fake_network_id',
                               'description': 'fake_name'}
        fake_logical_switch_list = [fake_logical_switch]
        fake_locator_dict = {'ovsdb_identifier': 'fake_ovsdb_id',
                             'uuid': 'fake_locator_id',
                             'dst_ip': 'fake_ip2'}
        fake_pl_dict = {'ovsdb_identifier': 'fake_ovsdb_id',
                        'dst_ip': 'fake_ip1'}
        fake_mac_dict = {'mac': 'fake_mac',
                         'uuid': 'fake_mac_id',
                         'locator': 'fake_locator_id2',
                         'logical_switch_id': 'fake_network_id',
                         'physical_locator_id': 'fake_locator_id1',
                         'ip_address': 'fake_ip1'}
        fake_dict = fake_mac_dict
        fake_dict['ovsdb_identifier'] = 'fake_ovsdb_id'
        core_plugin = mock.PropertyMock()
        type(self.service_plugin)._core_plugin = core_plugin
        (self.service_plugin._core_plugin.get_port.
         return_value) = {'device_owner': 'fake_owner',
                          'network_id': 'fake_network_id',
                          'mac_address': 'fake_mac'}
        with mock.patch.object(self.plugin,
                               '_get_ip_details',
                               return_value=(fake_ip1,
                                             fake_ip2)), \
            mock.patch.object(self.plugin,
                              '_get_network_details',
                              return_value=fake_network_dict) as get_network, \
            mock.patch.object(self.service_plugin,
                              'get_l2_gateway_connections',
                              return_value=fake_conn_list) as get_l2gw_conn, \
            mock.patch.object(self.plugin,
                              '_form_physical_locator_schema',
                              return_value=fake_locator_dict) as get_pl, \
            mock.patch.object(self.plugin,
                              '_get_dict',
                              return_value=fake_dict), \
            mock.patch.object(db,
                              'get_ucast_mac_remote_by_mac_and_ls',
                              return_value=fake_mac_dict) as get_ucast_mac, \
            mock.patch.object(self.plugin.agent_rpc,
                              'add_vif_to_gateway') as add_rpc, \
            mock.patch.object(self.plugin.agent_rpc,
                              'update_vif_to_gateway',
                              side_effect=RuntimeError) as update_rpc, \
            mock.patch.object(
                db,
                'add_pending_ucast_mac_remote') as add_pending_mac, \
            mock.patch.object(db, 'get_all_logical_switches_by_name',
                              return_value=fake_logical_switch_list):
            self.plugin.add_port_mac(self.context, fake_dict)
            get_network.assert_called_with(self.context, 'fake_network_id')
            get_l2gw_conn.assert_called_with(
                self.context, filters={'network_id': ['fake_network_id']})
            get_pl.assert_called_with(self.context, fake_pl_dict)
            get_ucast_mac.assert_called_with(self.context, fake_dict)
            self.assertFalse(add_rpc.called)
            self.assertTrue(update_rpc.called)
            self.assertTrue(add_pending_mac.called)

    def test_add_port_mac_tunnel_recreation(self):
        "Test case to test recreation of tunnels"
        "when the openvswitch agent is restarted."
        fake_ip1 = "fake_ip1"
        fake_ip2 = "fake_ip2"
        fake_network_dict = {'provider:segmentation_id': 100,
                             'name': 'fake_name',
                             'id': 'fake_network_id'}
        fake_conn_dict = {'id': 'fake_conn_id',
                          'l2_gateway_id': 'fake_gateway_id'}
        fake_conn_list = [fake_conn_dict]
        fake_logical_switch = {'ovsdb_identifier': 'fake_ovsdb_id',
                               'uuid': 'fake_network_id',
                               'description': 'fake_name'}
        fake_logical_switch_list = [fake_logical_switch]
        fake_locator_dict = {'ovsdb_identifier': 'fake_ovsdb_id',
                             'uuid': 'fake_locator_id',
                             'dst_ip': 'fake_ip2'}
        fake_pl_dict = {'ovsdb_identifier': 'fake_ovsdb_id',
                        'dst_ip': 'fake_ip1'}
        fake_mac_dict = {'mac': 'fake_mac',
                         'uuid': 'fake_mac_id',
                         'locator': 'fake_locator_id',
                         'logical_switch_id': 'fake_network_id',
                         'physical_locator_id': 'fake_locator_id',
                         'ip_address': 'fake_ip1'}
        fake_dict = fake_mac_dict
        fake_dict['ovsdb_identifier'] = 'fake_ovsdb_id'
        core_plugin = mock.PropertyMock()
        type(self.service_plugin)._core_plugin = core_plugin
        (self.service_plugin._core_plugin.get_port.
         return_value) = {'device_owner': 'fake_owner',
                          'network_id': 'fake_network_id',
                          'mac_address': 'fake_mac'}
        with mock.patch.object(self.plugin,
                               '_get_ip_details',
                               return_value=(fake_ip1,
                                             fake_ip2)), \
            mock.patch.object(self.plugin,
                              '_get_network_details',
                              return_value=fake_network_dict) as get_network, \
            mock.patch.object(self.service_plugin,
                              'get_l2_gateway_connections',
                              return_value=fake_conn_list) as get_l2gw_conn, \
            mock.patch.object(self.plugin,
                              '_form_physical_locator_schema',
                              return_value=fake_locator_dict) as get_pl, \
            mock.patch.object(self.plugin,
                              '_get_dict',
                              return_value=fake_dict), \
            mock.patch.object(db,
                              'get_ucast_mac_remote_by_mac_and_ls',
                              return_value=fake_mac_dict) as get_ucast_mac, \
            mock.patch.object(agent_api.L2gatewayAgentApi,
                              'add_vif_to_gateway') as add_rpc, \
            mock.patch.object(data.L2GatewayOVSDBCallbacks,
                              'get_ovsdbdata_object') as get_ovsdbdata_obj, \
            mock.patch.object(db,
                              'get_all_logical_switches_by_name',
                              return_value=fake_logical_switch_list):
            self.plugin.add_port_mac(self.context, fake_dict)
            get_network.assert_called_with(self.context, 'fake_network_id')
            get_l2gw_conn.assert_called_with(
                self.context, filters={'network_id': ['fake_network_id']})
            get_pl.assert_called_with(self.context, fake_pl_dict)
            get_ucast_mac.assert_called_with(self.context, fake_dict)
            self.assertFalse(add_rpc.called)
            self.assertTrue(get_ovsdbdata_obj.called)

    @mock.patch.object(db,
                       'get_all_logical_switches_by_name')
    @mock.patch.object(db,
                       'get_ucast_mac_remote_by_mac_and_ls',
                       return_value=True)
    def test_delete_port_mac_with_list(self,
                                       get_mac, get_ls):
        network_id = 'fake_network_id'
        fake_port_dict = {'network_id': 'fake_network_id',
                          'device_owner': 'fake_owner',
                          'mac_address': 'fake_mac',
                          'ovsdb_identifier': 'fake_ovsdb_id'}
        fake_logical_switch_dict = {'uuid': 'fake_uuid',
                                    'ovsdb_identifier': 'fake_ovsdb_id'}
        fake_logical_switch_list = [fake_logical_switch_dict]
        fake_dict = {'mac': 'fake_mac',
                     'logical_switch_uuid': 'fake_uuid',
                     'ovsdb_identifier': 'fake_ovsdb_id'}
        get_ls.return_value = fake_logical_switch_list
        with mock.patch.object(self.plugin.agent_rpc,
                               'delete_vif_from_gateway') as delete_rpc, \
            mock.patch.object(self.service_plugin,
                              'get_l2_gateway_connections',
                              return_value=True):
            self.plugin.delete_port_mac(self.context, fake_port_dict)
            get_ls.assert_called_with(self.context, network_id)
            get_mac.assert_called_with(self.context, fake_dict)
            delete_rpc.assert_called_with(
                self.context, 'fake_ovsdb_id', 'fake_uuid', ['fake_mac'])

    @mock.patch.object(db,
                       'get_logical_switch_by_name')
    @mock.patch.object(db,
                       'get_all_logical_switches_by_name')
    @mock.patch.object(db,
                       'get_ucast_mac_remote_by_mac_and_ls',
                       return_value=True)
    def test_delete_port_mac_for_single_l2gw_connection(self,
                                                        get_mac, get_ls,
                                                        get_ls_by_name):
        fake_port_dict = {'network_id': 'fake_network_id',
                          'device_owner': 'fake_owner',
                          'mac_address': 'fake_mac',
                          'ovsdb_identifier': 'fake_ovsdb_id'}
        fake_port_list = [fake_port_dict]
        fake_rec_dict = {'uuid': 'fake_network_id',
                         'ovsdb_identifier': 'fake_ovsdb_id'}
        fake_dict = {'logical_switch_name': 'fake_network_id',
                     'ovsdb_identifier': 'fake_ovsdb_id'}
        fake_ucast_mac_and_ls = {'mac': 'fake_mac',
                                 'logical_switch_uuid': 'fake_network_id',
                                 'ovsdb_identifier': 'fake_ovsdb_id'}
        get_ls_by_name.return_value = fake_rec_dict
        with mock.patch.object(self.plugin.agent_rpc,
                               'delete_vif_from_gateway') as delete_rpc, \
            mock.patch.object(self.service_plugin,
                              'get_l2_gateway_connections',
                              return_value=[1]):
            self.plugin.delete_port_mac(self.context, fake_port_list)
            get_ls_by_name.assert_called_with(self.context, fake_dict)
            get_ls.assert_not_called()
            get_mac.assert_called_with(self.context, fake_ucast_mac_and_ls)
            delete_rpc.assert_called_with(
                self.context, 'fake_ovsdb_id', 'fake_network_id', ['fake_mac'])

    @mock.patch.object(db,
                       'get_logical_switch_by_name')
    @mock.patch.object(db,
                       'get_all_logical_switches_by_name')
    @mock.patch.object(db,
                       'get_ucast_mac_remote_by_mac_and_ls',
                       return_value=True)
    def test_delete_port_mac_for_multiple_l2gw_connection(self,
                                                          get_mac, get_ls,
                                                          get_ls_by_name):
        fake_port_dict = {'network_id': 'fake_network_id',
                          'device_owner': 'fake_owner',
                          'mac_address': 'fake_mac',
                          'ovsdb_identifier': 'fake_ovsdb_id'}
        fake_port_list = [fake_port_dict]
        fake_rec_dict = {'logical_switch_name': 'fake_network_id',
                         'ovsdb_identifier': 'fake_ovsdb_id'}
        with mock.patch.object(self.plugin.agent_rpc,
                               'delete_vif_from_gateway') as delete_rpc, \
            mock.patch.object(self.service_plugin,
                              'get_l2_gateway_connections',
                              return_value=[1, 2]):
            self.plugin.delete_port_mac(self.context, fake_port_list)
            get_ls_by_name.assert_called_with(self.context, fake_rec_dict)
            get_ls.assert_not_called()
            get_mac.assert_not_called()
            delete_rpc.assert_not_called()

    @mock.patch.object(db,
                       'get_all_logical_switches_by_name')
    @mock.patch.object(db,
                       'get_ucast_mac_remote_by_mac_and_ls',
                       return_value=True)
    @mock.patch.object(db,
                       'get_all_vlan_bindings_by_logical_switch',
                       return_value=[1])
    @mock.patch.object(db,
                       'get_logical_switch_by_name')
    def test_delete_port_mac(self, get_ls, get_vlan_binding, get_mac,
                             get_all_ls):
        fake_port_list = [{'network_id': 'fake_network_id',
                           'device_owner': 'fake_owner',
                           'mac_address': 'fake_mac',
                           'allowed_address_pairs': [{'mac_address':
                                                      'fake_pairs'}],
                           'ovsdb_identifier': 'fake_ovsdb_id'}]
        fake_logical_switch_dict = {'uuid': 'fake_uuid',
                                    'ovsdb_identifier': 'fake_ovsdb_id'}
        lg_dict = {'logical_switch_name': 'fake_network_id',
                   'ovsdb_identifier': 'fake_ovsdb_id'}
        fake_dict = {'mac': 'fake_mac',
                     'logical_switch_uuid': 'fake_uuid',
                     'ovsdb_identifier': 'fake_ovsdb_id'}
        get_all_ls.return_value = fake_logical_switch_dict
        get_ls.return_value = fake_logical_switch_dict
        with mock.patch.object(self.plugin.agent_rpc,
                               'delete_vif_from_gateway') as delete_rpc:
            self.plugin.delete_port_mac(self.context, fake_port_list)
            self.assertFalse(get_all_ls.called)
            get_ls.assert_called_with(self.context, lg_dict)
            get_mac.assert_called_with(self.context, fake_dict)
            delete_rpc.assert_called_with(
                self.context, 'fake_ovsdb_id', 'fake_uuid',
                ['fake_pairs', 'fake_mac'])

    @mock.patch.object(db,
                       'get_all_logical_switches_by_name')
    @mock.patch.object(db,
                       'get_ucast_mac_remote_by_mac_and_ls',
                       return_value=True)
    @mock.patch.object(db, 'add_pending_ucast_mac_remote')
    @mock.patch.object(db, 'get_logical_switch_by_name')
    def test_delete_port_mac_with_ovsdb_server_down(self,
                                                    get_ls, add_pending_mac,
                                                    get_mac, get_all_ls):
        "Test case to test delete_port_mac when the OVSDB server is down."
        fake_port_list = [{'network_id': 'fake_network_id',
                           'device_owner': 'fake_owner',
                           'mac_address': 'fake_mac',
                           'ovsdb_identifier': 'fake_ovsdb_id'}]
        fake_logical_switch_dict = {'uuid': 'fake_uuid',
                                    'ovsdb_identifier': 'fake_ovsdb_id'}
        lg_dict = {'logical_switch_name': 'fake_network_id',
                   'ovsdb_identifier': 'fake_ovsdb_id'}
        fake_dict = {'mac': 'fake_mac',
                     'logical_switch_uuid': 'fake_uuid',
                     'ovsdb_identifier': 'fake_ovsdb_id'}
        get_all_ls.return_value = fake_logical_switch_dict
        get_ls.return_value = fake_logical_switch_dict
        with mock.patch.object(self.plugin.agent_rpc,
                               'delete_vif_from_gateway',
                               side_effect=RuntimeError) as delete_rpc:
            self.plugin.delete_port_mac(self.context, fake_port_list)
            self.assertFalse(get_all_ls.called)
            get_ls.assert_called_with(self.context, lg_dict)
            get_mac.assert_called_with(self.context, fake_dict)
            delete_rpc.assert_called_with(
                self.context, 'fake_ovsdb_id', 'fake_uuid', ['fake_mac'])
            self.assertTrue(add_pending_mac.called)

    def test_delete_l2_gateway_connection(self):
        self.db_context = ctx.get_admin_context()
        fake_conn_dict = {'l2_gateway_id': 'fake_l2gw_id',
                          'ovsdb_identifier': 'fake_ovsdb_id',
                          'network_id': 'fake_network_id'}
        ovsdb_id = 'fake_ovsdb_id'
        logical_switch = {'uuid': 'fake_uuid',
                          'ovsdb_identifier': 'fake_ovsdb_id'}
        fake_device_dict = {'id': 'fake_device_id',
                            'device_name': 'fake_device_name'}
        fake_device_list = [fake_device_dict]
        fake_identifier_list = ['fake_ovsdb_id']
        fake_ovsdb_list = ['fake_ovsdb_id']
        fake_port_dict = {'network_id': 'fake_network_id',
                          'device_owner': 'fake_owner',
                          'mac_address': 'fake_mac',
                          'ovsdb_identifier': 'fake_ovsdb_id'}
        DELETE = "DELETE"
        with mock.patch.object(self.service_plugin,
                               '_admin_check',
                               return_value=True) as admin_check, \
            mock.patch.object(self.service_plugin,
                              'get_l2_gateway_connection',
                              return_value=fake_conn_dict) as get_con, \
            mock.patch.object(
                self.plugin,
                '_get_identifer_list',
                return_value=fake_identifier_list) as get_id_list, \
            mock.patch.object(self.plugin,
                              '_get_set_of_ovsdb_ids',
                              return_value=fake_ovsdb_list) as get_ovsdb_id, \
            mock.patch.object(self.service_plugin,
                              'get_l2gateway_devices_by_gateway_id',
                              return_value=fake_device_list) as get_devices, \
            mock.patch.object(self.plugin,
                              '_process_port_list',
                              return_value=(ovsdb_id, logical_switch,
                                            fake_port_dict)) as port_list, \
            mock.patch.object(self.plugin.agent_rpc,
                              'update_connection_to_gateway') as update_rpc, \
            mock.patch.object(self.plugin,
                              '_remove_vm_macs') as remove_vm_mac:
            self.plugin.delete_l2_gateway_connection(self.context,
                                                     fake_conn_dict)
            admin_check.assert_called_with(self.context, 'DELETE')
            get_con.assert_called_with(self.context, fake_conn_dict)
            get_id_list.assert_called_with(self.context, fake_conn_dict)
            get_ovsdb_id.assert_called_with(
                self.context, fake_conn_dict,
                fake_identifier_list)
            get_devices.assert_called_with(self.context, 'fake_l2gw_id')
            port_list.assert_called_with(
                self.context, fake_device_dict,
                fake_conn_dict, DELETE, fake_identifier_list)
            self.assertTrue(update_rpc.called)
            remove_vm_mac.assert_called_with(
                self.context, 'fake_network_id', fake_ovsdb_list)

    def test_create_l2gateway_connection_with_switch_fault_status_down(self):
        self.db_context = ctx.get_admin_context()
        fake_l2gw_conn_dict = {'l2_gateway_connection': {
            'id': 'fake_id', 'network_id': 'fake_network_id',
            'l2_gateway_id': 'fake_l2gw_id'}}
        fake_device = {'devices': [{'device_name': 'fake_device',
                       'interfaces': [{'name': 'fake_interface'}]}]}
        fake_physical_port = {'uuid': 'fake_id',
                              'name': 'fake_name',
                              'physical_switch_id': 'fake_switch1',
                              'port_fault_status': 'UP'}
        fake_physical_switch = {'uuid': 'fake_id',
                                'name': 'fake_name',
                                'tunnel_ip': 'fake_tunnel_ip',
                                'ovsdb_identifier': 'fake_ovsdb_id',
                                'switch_fault_status': 'DOWN'}
        with mock.patch.object(self.service_plugin,
                               '_admin_check',
                               return_value=True), \
            mock.patch.object(self.service_plugin,
                              'get_l2_gateway', return_value=fake_device), \
            mock.patch.object(db, 'get_physical_port_by_name_and_ps',
                              return_value=fake_physical_port), \
            mock.patch.object(db, 'get_physical_switch_by_name',
                              return_value=fake_physical_switch), \
            mock.patch.object(self.service_plugin,
                              '_get_network',
                              return_value=True), \
            mock.patch.object(self.service_plugin,
                              '_get_l2_gateway',
                              return_value=True):
            self.assertRaises(l2gw_exc.L2GatewayPhysicalSwitchFaultStatus,
                              self.plugin.create_l2_gateway_connection,
                              self.db_context,
                              fake_l2gw_conn_dict)

    def test_create_l2gateway_connection_with_port_fault_status_down(self):
        self.db_context = ctx.get_admin_context()
        fake_l2gw_conn_dict = {'l2_gateway_connection': {
            'id': 'fake_id', 'network_id': 'fake_network_id',
            'l2_gateway_id': 'fake_l2gw_id'}}
        fake_device = {'devices': [{'device_name': 'fake_device',
                       'interfaces': [{'name': 'fake_interface'}]}]}
        fake_physical_port = {'uuid': 'fake_id',
                              'name': 'fake_name',
                              'physical_switch_id': 'fake_switch1',
                              'port_fault_status': 'DOWN'}
        fake_physical_switch = {'uuid': 'fake_id',
                                'name': 'fake_name',
                                'tunnel_ip': 'fake_tunnel_ip',
                                'ovsdb_identifier': 'fake_ovsdb_id',
                                'switch_fault_status': 'UP'}
        with mock.patch.object(self.service_plugin,
                               '_admin_check',
                               return_value=True), \
            mock.patch.object(self.service_plugin,
                              'get_l2_gateway', return_value=fake_device), \
            mock.patch.object(db, 'get_physical_port_by_name_and_ps',
                              return_value=fake_physical_port), \
            mock.patch.object(db, 'get_physical_switch_by_name',
                              return_value=fake_physical_switch), \
            mock.patch.object(self.service_plugin,
                              '_get_network',
                              return_value=True), \
            mock.patch.object(self.service_plugin,
                              '_get_l2_gateway',
                              return_value=True):
            self.assertRaises(l2gw_exc.L2GatewayPhysicalPortFaultStatus,
                              self.plugin.create_l2_gateway_connection,
                              self.db_context, fake_l2gw_conn_dict)

    @mock.patch.object(db, 'get_physical_port_by_name_and_ps')
    @mock.patch.object(db, 'get_physical_switch_by_name')
    def test_check_port_fault_status_and_switch_fault_status(
            self, phy_switch, phy_port):
        fake_device = {'devices': [{'device_name': 'fake_device',
                       'interfaces': [{'name': 'fake_interface'}]}]}
        self.db_context = ctx.get_admin_context()
        fake_device = {'devices': [{'device_name': 'fake_device',
                       'interfaces': [{'name': 'fake_interface'}]}]}
        fake_physical_port = {'uuid': 'fake_id',
                              'name': 'fake_name',
                              'physical_switch_id': 'fake_switch1',
                              'port_fault_status': None}
        fake_physical_switch = {'uuid': 'fake_id',
                                'name': 'fake_name',
                                'tunnel_ip': 'fake_tunnel_ip',
                                'ovsdb_identifier': 'fake_ovsdb_id',
                                'switch_fault_status': None}
        phy_port.return_value = fake_physical_port
        phy_switch.return_value = fake_physical_switch
        with mock.patch.object(self.service_plugin,
                               'get_l2_gateway',
                               return_value=fake_device) as get_l2gw:
            self.plugin._check_port_fault_status_and_switch_fault_status(
                mock.Mock(), mock.Mock())
            self.assertTrue(get_l2gw.called)
            self.assertTrue(phy_port.called)
            self.assertTrue(phy_switch.called)

    def test_create_l2_gateway_connection(self):
        self.db_context = ctx.get_admin_context()
        fake_l2gw_conn_dict = {'l2_gateway_connection': {
            'id': 'fake_id', 'network_id': 'fake_network_id',
            'l2_gateway_id': 'fake_l2gw_id'}}
        fake_port = {'device_owner': 'fake_owner',
                     'network_id': 'fake_network_id',
                     'mac_address': 'fake_mac',
                     'ip_address': 'fake_ip2',
                     'allowed_address_pairs': [{'ip_address': 'fake_ip2',
                                                'mac_address': 'fake_mac2'}],
                     }
        fake_port_list = [fake_port]
        fake_conn_dict = fake_l2gw_conn_dict.get('l2_gateway_connection')
        ovsdb_id = 'fake_ovsdb_id'
        logical_switch = {'uuid': 'fake_id'}
        fake_device_dict = {'device_name': 'fake_device_name'}
        fake_device_list = [fake_device_dict]
        fake_ls_dict = {'logical_switch_name': 'fake_network_id',
                        'ovsdb_identifier': 'fake_ovsdb_id'}
        fake_pl_dict = {'uuid': 'fake_uuid', 'dst_ip': 'fake_ip1',
                        'ovsdb_identifier': 'fake_ovsdb_id',
                        'macs': [fake_port]}
        fake_pl_list = [fake_pl_dict]
        with mock.patch.object(self.service_plugin,
                               '_admin_check',
                               return_value=True) as admin_check, \
            mock.patch.object(self.plugin,
                              '_validate_connection'), \
            mock.patch.object(self.service_plugin,
                              'get_l2gateway_devices_by_gateway_id',
                              return_value=fake_device_list) as get_devices, \
            mock.patch.object(self.plugin,
                              '_process_port_list',
                              return_value=(ovsdb_id,
                                            logical_switch,
                                            fake_port)) as port_list, \
            mock.patch.object(self.plugin,
                              '_get_logical_switch_dict',
                              return_value=fake_ls_dict) as get_ls, \
            mock.patch.object(self.plugin,
                              '_get_port_details',
                              return_value=fake_port_list) as get_port, \
            mock.patch.object(
                self.plugin,
                '_get_ip_details',
                return_value=('fake_ip1', 'fake_ip2')) as get_ip, \
            mock.patch.object(self.plugin,
                              '_get_dict',
                              return_value=mock.ANY) as get_dict, \
            mock.patch.object(
                db,
                'get_ucast_mac_remote_by_mac_and_ls') as get_ucast_mac, \
            mock.patch.object(self.plugin,
                              '_get_locator_list',
                              return_value=fake_pl_list) as get_pl, \
            mock.patch.object(self.plugin.agent_rpc,
                              'update_connection_to_gateway') as update_rpc:
            self.plugin.create_l2_gateway_connection(self.db_context,
                                                     fake_l2gw_conn_dict)
            admin_check.assert_called_with(self.db_context, 'CREATE')
            get_devices.assert_called_with(self.db_context, 'fake_l2gw_id')
            port_list.assert_called_with(
                self.db_context, fake_device_dict,
                fake_conn_dict, "CREATE")
            get_ls.assert_called_with(self.db_context,
                                      logical_switch,
                                      fake_conn_dict)
            get_port.assert_called_with(self.db_context, 'fake_network_id')
            self.assertTrue(get_ip.called)
            self.assertTrue(get_dict.called)
            self.assertEqual(get_ucast_mac.call_count, 2)
            self.assertTrue(get_pl.called)
            self.assertTrue(update_rpc.called)

    @mock.patch.object(db, 'get_physical_switch_by_name')
    def test_create_l2gateway_connection_with_invalid_device(self,
                                                             phy_switch):
        self.db_context = ctx.get_admin_context()
        fake_l2gw_conn_dict = {'l2_gateway_connection': {
            'id': 'fake_id', 'network_id': 'fake_network_id',
            'l2_gateway_id': 'fake_l2gw_id'}}
        fake_device = {'devices': [{'device_name': 'fake_device',
                       'interfaces': [{'name': 'fake_interface'}]}]}
        fake_physical_switch = None
        phy_switch.return_value = fake_physical_switch
        with mock.patch.object(self.service_plugin,
                               '_admin_check',
                               return_value=True), \
            mock.patch.object(self.service_plugin,
                              'get_l2_gateway',
                              return_value=fake_device), \
            mock.patch.object(self.service_plugin,
                              '_get_network',
                              return_value=True), \
            mock.patch.object(self.service_plugin,
                              '_get_l2_gateway',
                              return_value=True):
            self.assertRaises(l2gw_exc.L2GatewayDeviceNotFound,
                              self.plugin.create_l2_gateway_connection,
                              self.db_context, fake_l2gw_conn_dict)

    @mock.patch.object(db, 'get_physical_switch_by_name')
    @mock.patch.object(db, 'get_physical_port_by_name_and_ps')
    def test_create_l2gateway_connection_with_invalid_interface(
            self, phy_port, phy_switch):
        self.db_context = ctx.get_admin_context()
        fake_l2gw_conn_dict = {'l2_gateway_connection': {
            'id': 'fake_id', 'network_id': 'fake_network_id',
            'l2_gateway_id': 'fake_l2gw_id'}}
        fake_device = {'devices': [{'device_name': 'fake_device',
                       'interfaces': [{'name': 'fake_interface'}]}]}
        fake_physical_port = None
        fake_physical_switch = {'uuid': 'fake_id',
                                'name': 'fake_device',
                                'tunnel_ip': 'fake_tunnel_ip',
                                'ovsdb_identifier': 'fake_ovsdb_id',
                                'switch_fault_status': None}
        phy_switch.return_value = fake_physical_switch
        phy_port.return_value = fake_physical_port
        with mock.patch.object(self.service_plugin,
                               '_admin_check',
                               return_value=True), \
            mock.patch.object(self.service_plugin,
                              'get_l2_gateway',
                              return_value=fake_device), \
            mock.patch.object(self.service_plugin,
                              '_get_network',
                              return_value=True), \
            mock.patch.object(self.service_plugin,
                              '_get_l2_gateway',
                              return_value=True):
            self.assertRaises(l2gw_exc.L2GatewayInterfaceNotFound,
                              self.plugin.create_l2_gateway_connection,
                              self.db_context, fake_l2gw_conn_dict)

    def test_validate_gateway_for_update_with_invalid_device(self):
        self.db_context = ctx.get_admin_context()
        fake_l2gw_dict = {"l2_gateway": {"name": "fake_name",
                                         "devices":
                                         [{"interfaces":
                                           [{"name": "port1",
                                             "segmentation_id": [
                                                 "111"]}],
                                           "device_name": "device_name"}]}}
        with mock.patch.object(db,
                               'get_physical_switch_by_name',
                               return_value=None):
            self.assertRaises(l2gw_exc.L2GatewayDeviceNotFound,
                              self.plugin._validate_gateway_for_update,
                              self.db_context, fake_l2gw_dict)

    def test_validate_gateway_for_update_with_invalid_port(self):
        self.db_context = ctx.get_admin_context()
        fake_l2gw_dict = {"l2_gateway": {"name": "fake_name",
                                         "devices":
                                         [{"interfaces":
                                           [{"name": "port1",
                                             "segmentation_id": [
                                                 "111"]}],
                                           "device_name": "device_name"}]}}
        fake_physical_switch = {'uuid': 'fake_id',
                                'name': 'fake_device_name',
                                'tunnel_ip': 'fake_tunnel_ip',
                                'ovsdb_identifier': 'fake_ovsdb_id',
                                'switch_fault_status': None}
        with mock.patch.object(db,
                               'get_physical_switch_by_name',
                               return_value=fake_physical_switch), \
            mock.patch.object(db,
                              'get_physical_port_by_name_and_ps',
                              return_value=None):
            self.assertRaises(l2gw_exc.L2GatewayPhysicalPortNotFound,
                              self.plugin._validate_gateway_for_update,
                              self.db_context, fake_l2gw_dict)

    def test_validate_gateway_for_update_without_device(self):
        self.db_context = ctx.get_admin_context()
        fake_l2gw_dict = {"l2_gateway": {"name": "fake_name"}}
        with mock.patch.object(db,
                               'get_physical_switch_by_name',
                               return_value=None):
            self.plugin._validate_gateway_for_update(self.db_context,
                                                     fake_l2gw_dict)

    def test_update_l2_gateway(self):
        self.db_context = ctx.get_admin_context()
        fake_l2gw_dict = {'id': 'fake_l2gw_id',
                          'tenant_id': 'fake_tenant_id',
                          "name": "fake_l2gw_name",
                          "devices":
                                [{"interfaces": [{"name": "port1",
                                                  "segmentation_id": ["111"]}],
                                  "device_name": "fake_device_name",
                                  'id': 'fake_device_id'}]}
        fake_l2gw_id = 'fake_l2gw_id'
        fake_device_dict = {'devices':
                            [{'device_name': 'fake_device',
                              'interfaces': [{'name': 'fake_interface'}]}]}
        fake_device_list = [fake_device_dict]
        fake_conn_dict = {'l2_gateway_id': 'fake_l2gw_id',
                          'ovsdb_identifier': 'fake_ovsdb_id',
                          'network_id': 'fake_network_id'}
        fake_conn_list = [fake_conn_dict]
        fake_vlan_dict = {'vlan': 100,
                          'logical_switch_uuid': 'fake_uuid'}
        fake_physical_port = ovsdb_schema.PhysicalPort(
            uuid='fake_uuid',
            name='fake_interface_name',
            phys_switch_id='fake_uuid',
            vlan_binding_dicts=None,
            port_fault_status=None)
        fake_phys_port_dict = fake_physical_port.__dict__
        fake_phys_port_dict['vlan_bindings'] = [fake_vlan_dict]
        fake_port_list = [fake_phys_port_dict]
        ovsdb_id = 'fake_ovsdb_id'
        logical_switch = {'uuid': 'fake_id'}
        with mock.patch.object(self.service_plugin,
                               '_admin_check',
                               return_value=True) as admin_check, \
            mock.patch.object(
                self.plugin,
                '_validate_gateway_for_update') as validate_gateway, \
            mock.patch.object(self.service_plugin,
                              'get_l2gateway_devices_by_gateway_id',
                              return_value=fake_device_list) as device_list, \
            mock.patch.object(self.service_plugin,
                              '_get_l2_gateway_connections',
                              return_value=fake_conn_list) as conn_list, \
            mock.patch.object(self.plugin,
                              '_process_port_list',
                              return_value=(ovsdb_id,
                                            logical_switch,
                                            fake_port_list)) as port_list:
            self.plugin.update_l2_gateway(
                self.db_context, fake_l2gw_id, fake_l2gw_dict)
            admin_check.assert_called_with(self.db_context, 'UPDATE')
            self.assertTrue(validate_gateway.called)
            device_list.assert_called_with(self.db_context, 'fake_l2gw_id')
            conn_list.assert_called_with(self.db_context)
            port_list.assert_called_with(
                self.db_context, fake_device_dict,
                fake_conn_dict, "UPDATE")
            self.assertEqual(self.plugin.port_dict_before_update,
                             fake_port_list)

    def test_update_l2_gateway_postcommit_with_add_port(self):
        self.db_context = ctx.get_admin_context()
        fake_l2gw_dict = {'id': 'fake_l2gw_id',
                          'tenant_id': 'fake_tenant_id',
                          "name": "fake_l2gw_name",
                          "devices":
                                [{"interfaces": [{"name": "port1",
                                                  "segmentation_id": ["111"]}],
                                  "device_name": "fake_device_name",
                                  'id': 'fake_device_id'}]}
        fake_device_dict = {'devices': [{'device_name': 'fake_device_name',
                                         'interfaces': [{'name': 'port1'}]}]}
        fake_device_list = [fake_device_dict]
        fake_conn_dict = {'l2_gateway_id': 'fake_l2gw_id',
                          'ovsdb_identifier': 'fake_ovsdb_id',
                          'network_id': 'fake_network_id'}
        fake_conn_list = [fake_conn_dict]
        fake_vlan_dict = {'vlan': 100,
                          'logical_switch_uuid': 'fake_ls_id'}
        fake_physical_port_1 = ovsdb_schema.PhysicalPort(
            uuid='fake_uuid_1',
            name='fake_interface_name_1',
            phys_switch_id='fake_uuid_1',
            vlan_binding_dicts=None,
            port_fault_status=None)
        fake_physical_port_2 = ovsdb_schema.PhysicalPort(
            uuid='fake_uuid_2',
            name='fake_interface_name_2',
            phys_switch_id='fake_uuid_2',
            vlan_binding_dicts=None,
            port_fault_status=None)
        fake_phys_port_dict_1 = fake_physical_port_1.__dict__
        fake_phys_port_dict_1['vlan_bindings'] = [fake_vlan_dict]
        fake_phys_port_dict_2 = fake_physical_port_2.__dict__
        fake_phys_port_dict_2['vlan_bindings'] = [fake_vlan_dict]
        fake_port_list_before_update = [fake_phys_port_dict_1]
        fake_port_list_after_update = [fake_phys_port_dict_1,
                                       fake_phys_port_dict_2]
        port_list_add = [fake_phys_port_dict_2]
        fake_port = {'device_owner': 'fake_owner',
                     'network_id': 'fake_network_id',
                     'mac_address': 'fake_mac',
                     'ip_address': 'fake_ip',
                     'allowed_address_pairs': [{'ip_address': 'fake_ip',
                                                'mac_address': 'fake_mac'}]}
        fake_port_list = [fake_port]
        ovsdb_id = 'fake_ovsdb_id'
        logical_switch = {'uuid': 'fake_ls_id'}
        fake_ls_dict = {'logical_switch_name': 'fake_network_id',
                        'ovsdb_identifier': 'fake_ovsdb_id'}
        fake_pl_dict = {'uuid': 'fake_uuid', 'dst_ip': 'fake_ip1',
                        'ovsdb_identifier': 'fake_ovsdb_id',
                        'macs': [fake_port]}
        mac_dict = {'fake_ip1': fake_pl_dict['macs']}
        fake_pl_list = [fake_pl_dict]
        with mock.patch.object(self.plugin,
                               'port_dict_before_update',
                               fake_port_list_before_update), \
            mock.patch.object(self.service_plugin,
                              '_admin_check',
                              return_value=True) as admin_check, \
            mock.patch.object(self.service_plugin,
                              'get_l2gateway_devices_by_gateway_id',
                              return_value=fake_device_list) as device_list, \
            mock.patch.object(self.service_plugin,
                              '_get_l2_gateway_connections',
                              return_value=fake_conn_list) as conn_list, \
            mock.patch.object(
                self.plugin,
                '_process_port_list',
                return_value=(ovsdb_id, logical_switch,
                              fake_port_list_after_update)) as port_list, \
            mock.patch.object(self.plugin,
                              '_get_logical_switch_dict',
                              return_value=fake_ls_dict) as get_ls, \
            mock.patch.object(self.plugin,
                              '_get_port_details',
                              return_value=fake_port_list) as get_port, \
            mock.patch.object(self.plugin,
                              '_get_ip_details',
                              return_value=('fake_ip1',
                                            'fake_ip2')) as get_ip, \
            mock.patch.object(self.plugin, '_get_dict',
                              return_value=mock.ANY) as get_dict, \
            mock.patch.object(
                db,
                'get_ucast_mac_remote_by_mac_and_ls') as get_ucast_mac, \
            mock.patch.object(self.plugin,
                              '_get_locator_list',
                              return_value=fake_pl_list) as get_pl, \
            mock.patch.object(self.plugin.agent_rpc,
                              'update_connection_to_gateway') as update_rpc:
            self.plugin.update_l2_gateway_postcommit(self.db_context,
                                                     fake_l2gw_dict)
            admin_check.assert_called_with(self.db_context, 'UPDATE')
            device_list.assert_called_with(self.db_context, 'fake_l2gw_id')
            conn_list.assert_called_with(self.db_context)
            port_list.assert_called_with(
                self.db_context, fake_device_dict,
                fake_conn_dict, "UPDATE")
            get_ls.assert_called_with(self.db_context,
                                      logical_switch,
                                      fake_conn_dict)
            get_port.assert_called_with(self.db_context, 'fake_network_id')
            self.assertTrue(get_ip.called)
            self.assertTrue(get_dict.called)
            self.assertEqual(get_ucast_mac.call_count, 2)
            self.assertTrue(get_pl.called)
            update_rpc.assert_called_with(self.db_context, ovsdb_id,
                                          fake_ls_dict, fake_pl_list, mac_dict,
                                          port_list_add, 'CREATE')

    def test_update_l2_gateway_postcommit_with_del_port(self):
        self.db_context = ctx.get_admin_context()
        fake_l2gw_dict = {'id': 'fake_l2gw_id',
                          'tenant_id': 'fake_tenant_id',
                          "name": "fake_l2gw_name",
                          "devices":
                              [{"interfaces": [{"name": "port1",
                                                "segmentation_id": ["111"]}],
                                "device_name": "fake_device_name",
                                'id': 'fake_device_id'}]}
        fake_device_dict = {'devices': [{'device_name': 'fake_device_name',
                                         'interfaces': [{'name': 'port1'}]}]}
        fake_device_list = [fake_device_dict]
        fake_conn_dict = {'l2_gateway_id': 'fake_l2gw_id',
                          'ovsdb_identifier': 'fake_ovsdb_id',
                          'network_id': 'fake_network_id'}
        fake_conn_list = [fake_conn_dict]
        fake_vlan_dict = {'vlan': 100,
                          'logical_switch_uuid': 'fake_ls_id'}
        fake_physical_port_1 = ovsdb_schema.PhysicalPort(
            uuid='fake_uuid_1',
            name='fake_interface_name_1',
            phys_switch_id='fake_uuid_1',
            vlan_binding_dicts=None,
            port_fault_status=None)
        fake_physical_port_2 = ovsdb_schema.PhysicalPort(
            uuid='fake_uuid_2',
            name='fake_interface_name_2',
            phys_switch_id='fake_uuid_2',
            vlan_binding_dicts=None,
            port_fault_status=None)
        fake_phys_port_dict_1 = fake_physical_port_1.__dict__
        fake_phys_port_dict_1['vlan_bindings'] = [fake_vlan_dict]
        fake_phys_port_dict_2 = fake_physical_port_2.__dict__
        fake_phys_port_dict_2['vlan_bindings'] = [fake_vlan_dict]
        fake_port_list_before_update = [fake_phys_port_dict_1,
                                        fake_phys_port_dict_2]
        fake_port_list_after_update = [fake_phys_port_dict_1]
        port_list_del = [fake_phys_port_dict_2]
        ovsdb_id = 'fake_ovsdb_id'
        logical_switch = {'uuid': 'fake_ls_id'}
        fake_ls_dict = {'logical_switch_name': 'fake_network_id',
                        'ovsdb_identifier': 'fake_ovsdb_id'}
        mac_dict = {}
        fake_pl_list = []
        with mock.patch.object(self.plugin,
                               'port_dict_before_update',
                               fake_port_list_before_update), \
            mock.patch.object(self.service_plugin,
                              '_admin_check',
                              return_value=True) as admin_check, \
            mock.patch.object(self.service_plugin,
                              'get_l2gateway_devices_by_gateway_id',
                              return_value=fake_device_list) as device_list, \
            mock.patch.object(self.service_plugin,
                              '_get_l2_gateway_connections',
                              return_value=fake_conn_list) as conn_list, \
            mock.patch.object(
                self.plugin,
                '_process_port_list',
                return_value=(ovsdb_id, logical_switch,
                              fake_port_list_after_update)) as port_list, \
            mock.patch.object(self.plugin,
                              '_get_logical_switch_dict',
                              return_value=fake_ls_dict) as get_ls, \
            mock.patch.object(self.plugin.agent_rpc,
                              'update_connection_to_gateway') as update_rpc:
            self.plugin.update_l2_gateway_postcommit(self.db_context,
                                                     fake_l2gw_dict)
            admin_check.assert_called_with(self.db_context, 'UPDATE')
            device_list.assert_called_with(self.db_context, 'fake_l2gw_id')
            conn_list.assert_called_with(self.db_context)
            port_list.assert_called_with(
                self.db_context, fake_device_dict,
                fake_conn_dict, "UPDATE")
            get_ls.assert_called_with(self.db_context,
                                      logical_switch,
                                      fake_conn_dict)
            update_rpc.assert_called_with(self.db_context, ovsdb_id,
                                          fake_ls_dict, fake_pl_list, mac_dict,
                                          port_list_del, 'DELETE')



# Copyright (c) 2015 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import socket
import ssl
from unittest import mock

import eventlet

from neutron.tests import base

from networking_l2gw.services.l2gateway.agent import l2gateway_config as conf
from networking_l2gw.services.l2gateway.agent.ovsdb import ovsdb_monitor
from networking_l2gw.services.l2gateway.common import config
from networking_l2gw.services.l2gateway.common import constants as n_const
from networking_l2gw.services.l2gateway.common import ovsdb_schema
from networking_l2gw.services.l2gateway import exceptions
from networking_l2gw.tests.unit.services.l2gateway.agent.ovsdb import (
    test_base_connection as base_test)

from oslo_config import cfg
from oslo_log import log as logging
from oslo_serialization import jsonutils

LOG = logging.getLogger(__name__)


class TestOVSDBMonitor(base.BaseTestCase):
    def setUp(self):
        super(TestOVSDBMonitor, self).setUp()

        self.conf = mock.patch.object(conf, 'L2GatewayConfig').start()
        config.register_ovsdb_opts_helper(cfg.CONF)
        self.callback = mock.Mock()
        cfg.CONF.set_override('max_connection_retries', 0, 'ovsdb')

        self.sock = mock.patch('socket.socket').start()
        self.ssl_sock = mock.patch.object(ssl, 'wrap_socket').start()
        self.greenthread = mock.patch.object(eventlet.greenthread,
                                             'spawn_n').start()
        self.l2gw_ovsdb = ovsdb_monitor.OVSDBMonitor(mock.Mock(),
                                                     self.conf,
                                                     self.callback)
        self.op_id = 'abcd'
        props = {'select': {'initial': True,
                            'insert': True,
                            'delete': True,
                            'modify': True}}
        self.monitor_message = {'id': self.op_id,
                                'method': 'monitor',
                                'params': [n_const.OVSDB_SCHEMA_NAME,
                                           None,
                                           {'Logical_Switch': [props],
                                            'Physical_Switch': [props],
                                            'Physical_Port': [props],
                                            'Ucast_Macs_Local': [props],
                                            'Ucast_Macs_Remote': [props],
                                            'Physical_Locator': [props],
                                            'Mcast_Macs_Local': [props],
                                            'Physical_Locator_Set': [props]}]}

        fake_message = {'Logical_Switch': props,
                        'Physical_Switch': props,
                        'Physical_Port': props,
                        'Ucast_Macs_Local': props,
                        'Ucast_Macs_Remote': props,
                        'Physical_Locator': props,
                        'Mcast_Macs_Local': props,
                        'Physical_Locator_Set': props}

        self.msg = self.monitor_message
        self.msg1 = {'result': fake_message}
        self.msg2 = {'method': 'update',
                     'params': ['', fake_message]}
        self.l2gw_ovsdb.responses = [self.monitor_message]

    def test_init(self):
        """Test case to test __init__."""

        fakesocket = base_test.SocketClass()
        with mock.patch.object(ovsdb_monitor.LOG, 'debug'), \
                mock.patch.object(eventlet.greenthread, 'spawn') as gt, \
                mock.patch.object(socket, 'socket',
                                  return_value=fakesocket):
            self.l2gw_ovsdb.__init__(mock.Mock(), self.conf,
                                     self.callback)
            self.assertTrue(self.l2gw_ovsdb.connected)
            self.assertTrue(gt.called)
            self.assertTrue(self.sock.called)

    def test_setup_dispatch_table(self):
        expected_dict = {'Logical_Switch':
                         self.l2gw_ovsdb._process_logical_switch,
                         'Ucast_Macs_Local':
                         self.l2gw_ovsdb._process_ucast_macs_local,
                         'Physical_Locator':
                         self.l2gw_ovsdb._process_physical_locator,
                         'Ucast_Macs_Remote':
                         self.l2gw_ovsdb._process_ucast_macs_remote,
                         'Mcast_Macs_Local':
                         self.l2gw_ovsdb._process_mcast_macs_local,
                         'Physical_Locator_Set':
                         self.l2gw_ovsdb._process_physical_locator_set
                         }
        self.l2gw_ovsdb._setup_dispatch_table()
        self.assertEqual(expected_dict, self.l2gw_ovsdb.dispatch_table)

    def test_set_monitor_response_handler(self):
        """Test case to test _set_monitor_response_handler with error_msg."""
        self.l2gw_ovsdb.connected = True
        with mock.patch.object(ovsdb_monitor.OVSDBMonitor,
                               '_set_handler') as set_handler, \
                mock.patch.object(ovsdb_monitor.OVSDBMonitor,
                                  'send', return_value=True) as send, \
                mock.patch.object(ovsdb_monitor.OVSDBMonitor,
                                  '_process_response',
                                  return_value=(mock.ANY, False)
                                  ) as process_resp, \
                mock.patch.object(ovsdb_monitor.OVSDBMonitor,
                                  '_process_monitor_msg'
                                  ) as process_monitor_msg, \
                mock.patch.object(ovsdb_monitor.LOG, 'warning'):
            self.l2gw_ovsdb.set_monitor_response_handler()
            self.assertTrue(set_handler.called)
            self.assertTrue(send.called)
            self.assertTrue(process_monitor_msg.called)
            self.assertTrue(process_resp.called)

    def test_set_monitor_response_handler_with_error_in_send(self):
        """Test case to test _set_monitor_response_handler."""
        self.l2gw_ovsdb.connected = True
        with mock.patch.object(ovsdb_monitor.OVSDBMonitor,
                               '_set_handler') as set_handler, \
            mock.patch.object(ovsdb_monitor.OVSDBMonitor,
                              'send', return_value=False) as send, \
            mock.patch.object(ovsdb_monitor.OVSDBMonitor,
                              '_process_response',
                              return_value=(mock.ANY, True)) as process_resp, \
            mock.patch.object(ovsdb_monitor.OVSDBMonitor,
                              '_process_monitor_msg') as process_monitor_msg, \
            mock.patch.object(ovsdb_monitor.LOG,
                              'warning'):
            self.l2gw_ovsdb.set_monitor_response_handler()
            self.assertTrue(set_handler.called)
            self.assertTrue(send.called)
            self.assertFalse(process_resp.called)
            self.assertFalse(process_monitor_msg.called)

    def test_update_event_handler(self):
        """Test case to test _update_event_handler."""
        with mock.patch.object(ovsdb_monitor.OVSDBMonitor,
                               '_process_update_event'
                               ) as process_update_event:
            self.l2gw_ovsdb._update_event_handler(self.msg, mock.ANY)
            process_update_event.assert_called_once_with(self.msg, mock.ANY)

    def test_process_update_event(self):
        """Test case to test _process_update_event."""
        with mock.patch.object(ovsdb_monitor.OVSDBMonitor,
                               '_process_physical_port'
                               ) as proc_phy_port, \
            mock.patch.object(ovsdb_monitor.OVSDBMonitor,
                              '_process_physical_switch'
                              ) as proc_phy_switch, \
            mock.patch.object(ovsdb_monitor.OVSDBMonitor,
                              '_process_logical_switch'
                              ) as proc_logic_switch, \
            mock.patch.object(ovsdb_monitor.OVSDBMonitor,
                              '_process_ucast_macs_local'
                              ) as proc_ucast_mac, \
            mock.patch.object(ovsdb_monitor.OVSDBMonitor,
                              '_process_physical_locator'
                              ) as proc_phy_loc, \
            mock.patch.object(ovsdb_monitor.OVSDBMonitor,
                              '_process_ucast_macs_remote'
                              ) as proc_ucast_mac_remote, \
            mock.patch.object(ovsdb_monitor.OVSDBMonitor,
                              '_process_mcast_macs_local'
                              ) as proc_mcast_mac_local, \
            mock.patch.object(
                ovsdb_monitor.OVSDBMonitor,
                '_process_physical_locator_set') as proc_phys_loc_set:
            self.l2gw_ovsdb._setup_dispatch_table()
            self.l2gw_ovsdb._process_update_event(self.msg2, mock.ANY)
            self.assertTrue(proc_phy_port.called)
            self.assertTrue(proc_phy_switch.called)
            self.assertTrue(proc_logic_switch.called)
            self.assertTrue(proc_ucast_mac.called)
            self.assertTrue(proc_phy_loc.called)
            self.assertTrue(proc_ucast_mac_remote.called)
            self.assertTrue(proc_mcast_mac_local.called)
            self.assertTrue(proc_phys_loc_set.called)
            self.assertTrue(self.callback.called)

    def test_process_response_raise_exception(self):
        """Test case to test _process_response with exception."""
        with mock.patch.object(ovsdb_monitor.OVSDBMonitor,
                               '_response') as resp:
            with mock.patch.object(ovsdb_monitor.LOG, 'debug'):
                self.assertRaises(exceptions.OVSDBError,
                                  self.l2gw_ovsdb._process_response,
                                  self.op_id)
                resp.assert_called_once_with(self.op_id)

    def test_process_response(self):
        """Test case to test _process_response."""
        with mock.patch.object(ovsdb_monitor.OVSDBMonitor,
                               '_response',
                               return_value={'key':
                                             'some_value'
                                             }) as resp:
            with mock.patch.object(ovsdb_monitor.LOG, 'debug'):
                self.l2gw_ovsdb._process_response(self.op_id)
                resp.assert_called_once_with(self.op_id)

    def test_default_echo_handler(self):
        """Test case to test _default_echo_handler."""
        dummy_msg = {'params': 'fake_params',
                     'id': 'fake_id'}
        with mock.patch.object(ovsdb_monitor.OVSDBMonitor,
                               'send') as send:
            self.l2gw_ovsdb._default_echo_handler(dummy_msg, mock.ANY)
            self.assertTrue(send.called)

    def test_set_handler(self):
        """Test case to test _set_handler."""
        dummy_method = "dummy"
        dummy_handler = "handler"
        self.l2gw_ovsdb._set_handler(dummy_method, dummy_handler)
        self.assertEqual(self.l2gw_ovsdb.handlers[dummy_method],
                         dummy_handler)

    def test_on_remote_message(self):
        """Test case to test _on_remote_message."""
        self.l2gw_ovsdb.handlers = mock.Mock()
        with mock.patch.object(ovsdb_monitor.LOG, 'debug'):
            with mock.patch.object(jsonutils, 'loads') as json_load:
                self.l2gw_ovsdb._on_remote_message(self.msg)
                self.assertTrue(json_load.called)
                handler_method = json_load.return_value
                handler_method.get.assert_called_once_with(mock.ANY, mock.ANY)

    def test_rcv_thread_none(self):
        """Test case to test _rcv_thread receives None from socket."""
        self.assertTrue(self.l2gw_ovsdb.read_on)
        with mock.patch.object(self.l2gw_ovsdb.socket,
                               'recv', return_value=None) as sock_recv:
            with mock.patch.object(self.l2gw_ovsdb.socket,
                                   'close') as sock_close:
                self.l2gw_ovsdb._rcv_thread()
                self.assertTrue(sock_recv.called)
                self.assertFalse(self.l2gw_ovsdb.connected)
                self.assertFalse(self.l2gw_ovsdb.read_on)
                self.assertTrue(sock_close.called)

    def test_rcv_thread_exception(self):
        """Test case to test _rcv_thread with exception."""
        with mock.patch.object(self.l2gw_ovsdb.socket, 'recv',
                               side_effect=Exception,
                               return_value=None) as sock_recv, \
                mock.patch.object(self.l2gw_ovsdb.socket,
                                  'close') as sock_close, \
                mock.patch.object(ovsdb_monitor.LOG,
                                  'exception') as logger_call:
            self.l2gw_ovsdb._rcv_thread()
            self.assertTrue(logger_call.called)
            self.assertTrue(sock_recv.called)
            self.assertFalse(self.l2gw_ovsdb.connected)
            self.assertFalse(self.l2gw_ovsdb.read_on)
            self.assertTrue(sock_close.called)

    def test_form_ovsdb_data(self):
        some_value = mock.Mock()
        expect = {n_const.OVSDB_IDENTIFIER: self.conf.ovsdb_identifier,
                  'new_logical_switches': some_value,
                  'new_physical_switches': some_value,
                  'new_physical_ports': some_value,
                  'new_physical_locators': some_value,
                  'new_local_macs': some_value,
                  'new_remote_macs': some_value,
                  'new_mlocal_macs': some_value,
                  'new_locator_sets': some_value,
                  'deleted_logical_switches': some_value,
                  'deleted_physical_switches': some_value,
                  'deleted_physical_ports': some_value,
                  'deleted_physical_locators': some_value,
                  'deleted_local_macs': some_value,
                  'deleted_remote_macs': some_value,
                  'deleted_mlocal_macs': some_value,
                  'deleted_locator_sets': some_value,
                  'modified_logical_switches': some_value,
                  'modified_physical_switches': some_value,
                  'modified_physical_ports': some_value,
                  'modified_physical_locators': some_value,
                  'modified_local_macs': some_value,
                  'modified_remote_macs': some_value,
                  'modified_mlocal_macs': some_value,
                  'modified_locator_sets': some_value}
        with mock.patch.object(ovsdb_monitor.OVSDBMonitor,
                               '_get_list',
                               return_value=some_value
                               ):
            result = self.l2gw_ovsdb._form_ovsdb_data(mock.Mock(), mock.ANY)
            self.assertEqual(expect, result)

    def test_process_physical_port(self):
        """Test case to process new physical_port."""
        fake_id = 'fake_id'
        add = {'new': {'uuid': 'fake_id',
                       'name': 'fake_name',
                       'physical_switch_id': 'fake_switch_id',
                       'port_fault_status': 'fake_status',
                       'vlan_bindings': [["some"], []]}}
        delete = {'old': {'uuid': 'fake_id_old',
                          'name': 'fake_name_old',
                          'physical_switch_id': 'fake_switch_id_old'}}
        modify = {}
        modify.update(add)
        modify.update(delete)
        port_map = {'fake_id': 'fake_switch_id'}
        data_dict = {'new_physical_ports': [],
                     'modified_physical_ports': [],
                     'deleted_physical_ports': []}

        with mock.patch.object(ovsdb_schema, 'PhysicalPort') as phy_port:
            with mock.patch.object(ovsdb_schema, 'VlanBinding'):
                # test add
                self.l2gw_ovsdb._process_physical_port(fake_id,
                                                       add,
                                                       port_map,
                                                       data_dict)
                phy_port.assert_called_once_with(
                    fake_id, 'fake_name', None, None, 'fake_status')
                self.assertIn(phy_port.return_value,
                              data_dict.get('new_physical_ports'))

                # test modify
                self.l2gw_ovsdb._process_physical_port(fake_id,
                                                       modify,
                                                       port_map,
                                                       data_dict)
                self.assertIn(phy_port.return_value,
                              data_dict.get('modified_physical_ports'))
                # test delete
                self.l2gw_ovsdb._process_physical_port(fake_id,
                                                       delete,
                                                       port_map,
                                                       data_dict)
                self.assertIn(phy_port.return_value,
                              data_dict.get('deleted_physical_ports'))

    def test_process_physical_port_with_empty_fault_status(self):
        """Test case to process new physical_port with empty fault status."""
        fake_id = 'fake_id'
        add = {'new': {'uuid': 'fake_id',
                       'name': 'fake_name',
                       'physical_switch_id': 'fake_switch_id',
                       'port_fault_status': ['set', []],
                       'vlan_bindings': [["some"], []]}}
        port_map = {'fake_id': 'fake_switch_id'}
        data_dict = {'new_physical_ports': [],
                     'modified_physical_ports': [],
                     'deleted_physical_ports': []}

        with mock.patch.object(ovsdb_schema, 'PhysicalPort') as phy_port:
            with mock.patch.object(ovsdb_schema, 'VlanBinding'):
                # test add
                self.l2gw_ovsdb._process_physical_port(fake_id,
                                                       add,
                                                       port_map,
                                                       data_dict)
                phy_port.assert_called_once_with(
                    fake_id, 'fake_name', None, None, None)
                self.assertIn(phy_port.return_value,
                              data_dict.get('new_physical_ports'))

    def test_process_physical_switch(self):
        """Test case to process new physical_switch."""
        port_map = {'fake_id': 'fake_switch_id'}
        with mock.patch.object(ovsdb_schema, 'PhysicalPort') as phy_port:
            with mock.patch.object(ovsdb_schema,
                                   'PhysicalSwitch') as phy_switch:
                physical_port = phy_port.return_value
                fake_id = 'fake_id'
                add = {'new': {'uuid': 'fake_id',
                               'name': 'fake_name',
                               'tunnel_ips': 'fake_tunnel_ip',
                               'switch_fault_status': 'fake_status',
                               'ports': ['set', 'set',
                                         physical_port]}}
                delete = {'old': {'uuid': 'fake_id_old',
                                  'name': 'fake_name_old',
                                  'tunnel_ips': 'fake_tunnel_ip_old',
                                  'ports': ['', physical_port]}}
                modify = {}
                modify.update(add)
                modify.update(delete)
                data_dict = {'new_physical_switches': [],
                             'modified_physical_switches': [],
                             'deleted_physical_switches': [],
                             'new_physical_ports': []}
                # test add
                self.l2gw_ovsdb._process_physical_switch(fake_id,
                                                         add,
                                                         port_map,
                                                         data_dict)
                self.assertIn(phy_switch.return_value,
                              data_dict['new_physical_switches'])
                phy_switch.assert_called_once_with(
                    'fake_id', 'fake_name',
                    'fake_tunnel_ip', 'fake_status')
                # test modify
                self.l2gw_ovsdb._process_physical_switch(fake_id,
                                                         modify,
                                                         port_map,
                                                         data_dict)
                self.assertIn(phy_switch.return_value,
                              data_dict['modified_physical_switches'])
                # test delete
                self.l2gw_ovsdb._process_physical_switch(fake_id,
                                                         delete,
                                                         port_map,
                                                         data_dict)
                self.assertIn(phy_switch.return_value,
                              data_dict['deleted_physical_switches'])

    def test_process_physical_switch_with_empty_fault_status(self):
        """Test case to process new physical_switch with empty fault status."""
        port_map = {'fake_id': 'fake_switch_id'}
        with mock.patch.object(ovsdb_schema, 'PhysicalPort') as phy_port:
            with mock.patch.object(ovsdb_schema,
                                   'PhysicalSwitch') as phy_switch:
                physical_port = phy_port.return_value
                fake_id = 'fake_id'
                add = {'new': {'uuid': 'fake_id',
                               'name': 'fake_name',
                               'tunnel_ips': 'fake_tunnel_ip',
                               'switch_fault_status': ['set', []],
                               'ports': ['set', 'set',
                                         physical_port]}}
                data_dict = {'new_physical_switches': [],
                             'modified_physical_switches': [],
                             'deleted_physical_switches': [],
                             'new_physical_ports': []}
                # test add
                self.l2gw_ovsdb._process_physical_switch(fake_id,
                                                         add,
                                                         port_map,
                                                         data_dict)
                self.assertIn(phy_switch.return_value,
                              data_dict['new_physical_switches'])
                phy_switch.assert_called_once_with('fake_id', 'fake_name',
                                                   'fake_tunnel_ip', None)

    def test_process_logical_switch(self):
        """Test case to process new logical_switch."""
        fake_id = 'fake_id'
        fake_name = 'fake_name'
        fake_tunnel_key = 'fake_tunnel_key'
        add = {'new': {'name': 'fake_name',
                       'tunnel_key': 'fake_tunnel_key'}}
        delete = {'old': {'name': 'fake_name_old',
                          'tunnel_key': 'fake_tunnel_key_old'}}
        modify = {}
        modify.update(add)
        modify.update(delete)
        data_dict = {'new_logical_switches': [],
                     'modified_logical_switches': [],
                     'deleted_logical_switches': []}

        with mock.patch.object(ovsdb_schema,
                               'LogicalSwitch') as logical_switch:
            # test add
            self.l2gw_ovsdb._process_logical_switch(fake_id, add,
                                                    data_dict)
            logical_switch.assert_called_once_with(
                fake_id, fake_name, fake_tunnel_key, None)
            self.assertIn(logical_switch.return_value,
                          data_dict['new_logical_switches'])

            # test modify
            self.l2gw_ovsdb._process_logical_switch(fake_id, modify,
                                                    data_dict)
            self.assertIn(logical_switch.return_value,
                          data_dict['modified_logical_switches'])

            # test delete
            self.l2gw_ovsdb._process_logical_switch(fake_id, delete,
                                                    data_dict)
            self.assertIn(logical_switch.return_value,
                          data_dict['deleted_logical_switches'])

    def test_process_ucast_macs_local(self):
        """Test case to process new ucast_macs_local."""
        fake_id = 'fake_id'
        add = {'new': {'locator': ["uuid", "fake_locator_id"],
                       'MAC': 'fake_mac',
                       'logical_switch':
                       ["uuid",
                        "fake_logical_switch_id"],
                       'ipaddr': 'fake_ip_address'}}
        delete = {'old': {'MAC': 'fake_mac_old',
                          'logical_switch':
                          ["uuid",
                           "fake_logical_switch_id_old"]}}
        modify = {}
        modify.update(add)
        modify.update(delete)
        fake_mac = 'fake_mac'
        fake_logical_switch_id = 'fake_logical_switch_id'
        fake_locator_id = 'fake_locator_id'
        fake_ip_address = 'fake_ip_address'
        data_dict = {'new_local_macs': [],
                     'modified_local_macs': [],
                     'deleted_local_macs': []}
        with mock.patch.object(ovsdb_schema,
                               'UcastMacsLocal') as ucast_mac_local:
            # test add
            self.l2gw_ovsdb._process_ucast_macs_local(fake_id, add,
                                                      data_dict)
            ucast_mac_local.assert_called_once_with(fake_id, fake_mac,
                                                    fake_logical_switch_id,
                                                    fake_locator_id,
                                                    fake_ip_address)
            self.assertIn(ucast_mac_local.return_value,
                          data_dict['new_local_macs'])

            # test modify
            self.l2gw_ovsdb._process_ucast_macs_local(fake_id, modify,
                                                      data_dict)
            self.assertIn(ucast_mac_local.return_value,
                          data_dict['modified_local_macs'])

            # test delete
            self.l2gw_ovsdb._process_ucast_macs_local(fake_id, delete,
                                                      data_dict)
            self.assertIn(ucast_mac_local.return_value,
                          data_dict['deleted_local_macs'])

    def test_process_ucast_macs_remote(self):
        """Test case to process new ucast_macs_remote."""
        fake_id = 'fake_id'
        add = {'new': {'locator': ["uuid", "fake_locator_id"],
                       'MAC': 'fake_mac',
                       'logical_switch':
                       ["uuid",
                        "fake_logical_switch_id"],
                       'ipaddr': 'fake_ip_address'}}
        delete = {'old': {'MAC': 'fake_mac_old',
                          'logical_switch':
                          ["uuid", "fake_logical_switch_id_old"]}}
        modify = {}
        modify.update(add)
        modify.update(delete)
        fake_mac = 'fake_mac'
        fake_logical_switch_id = 'fake_logical_switch_id'
        fake_locator_id = 'fake_locator_id'
        fake_ip_address = 'fake_ip_address'
        data_dict = {'new_remote_macs': [],
                     'modified_remote_macs': [],
                     'deleted_remote_macs': []}

        with mock.patch.object(ovsdb_schema,
                               'UcastMacsRemote') as ucast_mac_remote:
            # test add
            self.l2gw_ovsdb._process_ucast_macs_remote(fake_id,
                                                       add,
                                                       data_dict)
            ucast_mac_remote.assert_called_once_with(fake_id, fake_mac,
                                                     fake_logical_switch_id,
                                                     fake_locator_id,
                                                     fake_ip_address)
            self.assertIn(ucast_mac_remote.return_value,
                          data_dict['new_remote_macs'])
            # test modify
            self.l2gw_ovsdb._process_ucast_macs_remote(fake_id,
                                                       modify,
                                                       data_dict)
            self.assertIn(ucast_mac_remote.return_value,
                          data_dict['modified_remote_macs'])

            # test delete
            self.l2gw_ovsdb._process_ucast_macs_remote(fake_id,
                                                       delete,
                                                       data_dict)
            self.assertIn(ucast_mac_remote.return_value,
                          data_dict['deleted_remote_macs'])

    def test_process_physical_locator(self):
        """Test case to process new physical locator."""
        fake_id = 'fake_id'
        add = {'new': {"dst_ip": "fake_dst_ip"}}
        delete = {'old': {"dst_ip": "fake_dst_ip_old"}}
        modify = {}
        modify.update(add)
        modify.update(delete)
        data_dict = {'new_physical_locators': [],
                     'modified_physical_locators': [],
                     'deleted_physical_locators': []}

        fake_dst_ip = 'fake_dst_ip'
        with mock.patch.object(ovsdb_schema, 'PhysicalLocator') as phy_locator:
            # test add
            self.l2gw_ovsdb._process_physical_locator(fake_id, add,
                                                      data_dict)
            phy_locator.assert_called_once_with(fake_id, fake_dst_ip)
            self.assertIn(phy_locator.return_value,
                          data_dict['new_physical_locators'])
            # test delete
            self.l2gw_ovsdb._process_physical_locator(fake_id, delete,
                                                      data_dict)
            self.assertIn(phy_locator.return_value,
                          data_dict['deleted_physical_locators'])

            # test modify
            self.l2gw_ovsdb._process_physical_locator(fake_id, modify,
                                                      data_dict)
            self.assertIn(phy_locator.return_value,
                          data_dict['modified_physical_locators'])

    def test_process_mcast_macs_local(self):
        """Test case to process new mcast_macs_local."""
        fake_id = 'fake_id'
        add = {'new': {'locator_set': ["uuid", "fake_locator_id"],
                       'MAC': 'fake_mac',
                       'logical_switch': ["uuid",
                                          "fake_logical_switch_id"],
                       'ipaddr': 'fake_ip_address'}}
        delete = {'old': {'locator': ["uuid", "fake_locator_id"],
                          'MAC': 'fake_mac',
                          'logical_switch': ["uuid",
                                             "fake_logical_switch_id"],
                          'ipaddr': 'fake_ip_address'}}

        fake_mac = 'fake_mac'
        fake_logical_switch_id = 'fake_logical_switch_id'
        fake_locator_id = 'fake_locator_id'
        fake_ip_address = 'fake_ip_address'
        data_dict = {'new_mlocal_macs': [],
                     'deleted_mlocal_macs': []}
        with mock.patch.object(ovsdb_schema,
                               'McastMacsLocal') as mcast_mac_local:
            # test add
            self.l2gw_ovsdb._process_mcast_macs_local(fake_id,
                                                      add, data_dict)
            mcast_mac_local.assert_called_once_with(fake_id, fake_mac,
                                                    fake_logical_switch_id,
                                                    fake_locator_id,
                                                    fake_ip_address)
            self.assertIn(mcast_mac_local.return_value,
                          data_dict['new_mlocal_macs'])

            # test delete
            self.l2gw_ovsdb._process_mcast_macs_local(fake_id,
                                                      delete,
                                                      data_dict)
            self.assertTrue(mcast_mac_local.called)
            self.assertIn(mcast_mac_local.return_value,
                          data_dict['deleted_mlocal_macs'])

    def test_process_physical_locator_set(self):
        """Test case to process new physical_locaor_set."""
        fake_id = 'fake_id'
        add = {'new': {'locators': ["set", [["uuid", "fake_id1"],
                                            ["uuid", "fake_id2"]]]
                       }}
        delete = {'old': {'locators': ["set", [["uuid", "fake_id1"],
                                               ["uuid", "fake_id2"]]]
                          }}
        fake_locators = ["fake_id1", "fake_id2"]
        data_dict = {'new_locator_sets': [],
                     'deleted_locator_sets': []}
        with mock.patch.object(ovsdb_schema,
                               'PhysicalLocatorSet') as phys_loc_set:
            self.l2gw_ovsdb._process_physical_locator_set(fake_id,
                                                          add,
                                                          data_dict)
            phys_loc_set.assert_called_once_with(fake_id, fake_locators)
            PhysLocatorSet = phys_loc_set.return_value
            self.assertIn(PhysLocatorSet,
                          data_dict['new_locator_sets'])

            self.l2gw_ovsdb._process_physical_locator_set(fake_id,
                                                          delete,
                                                          data_dict)
            self.assertTrue(phys_loc_set.called)
            PhysLocatorSet = phys_loc_set.return_value
            self.assertIn(PhysLocatorSet,
                          data_dict['deleted_locator_sets'])


class SocketClass(object):
    def __init__(self,
                 connect_error=None,
                 send_error=None,
                 recv_error=None,
                 rcv_data=None,
                 sock=None,
                 ip_addr=None):
        self.connect_error = connect_error
        self.rcv_data = rcv_data
        self.send_error = send_error
        self.recv_error = recv_error
        self.sock = sock
        self.ip_addr = ip_addr

    def connect(self, ip_port):
        if self.connect_error:
            raise self.connect_error

    def send(self, data):
        if self.send_error:
            raise self.send_error
        return len(data)

    def recv(self, length):
        if self.recv_error:
            raise self.recv_error
        return self.rcv_data

    def bind(self, host_port):
        pass

    def listen(self, conn):
        pass

    def accept(self):
        return self.sock, self.ip_addr

    def setsockopt(self, sock_opt, sock_reuse, mode):
        pass


class TestOVSDBMonitor_with_enable_manager(base.BaseTestCase):
    def setUp(self):
        super(TestOVSDBMonitor_with_enable_manager, self).setUp()
        config.register_ovsdb_opts_helper(cfg.CONF)
        cfg.CONF.set_override('enable_manager', True, 'ovsdb')
        self.conf = mock.Mock()
        self.callback = mock.Mock()
        self.l2gw_ovsdb = ovsdb_monitor.OVSDBMonitor(mock.Mock(),
                                                     self.conf,
                                                     self.callback)

    def test_init_with_enable_manager(self):
        self.l2gw_ovsdb.__init__(mock.Mock(), self.conf,
                                 self.callback)
        self.assertTrue(self.l2gw_ovsdb.enable_manager)
        self.assertFalse(self.l2gw_ovsdb.check_monitor_table_thread)



# Copyright (c) 2015 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from oslo_log import log as logging
from oslo_utils import timeutils
from sqlalchemy import asc
from sqlalchemy.orm import exc

from neutron_lib.db import api as db_api

from networking_l2gw.db.l2gateway.ovsdb import models

LOG = logging.getLogger(__name__)


def add_vlan_binding(context, record_dict):
    """Insert a vlan binding of a given physical port."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        binding = models.VlanBindings(
            port_uuid=record_dict['port_uuid'],
            vlan=record_dict['vlan'],
            logical_switch_uuid=record_dict['logical_switch_uuid'],
            ovsdb_identifier=record_dict['ovsdb_identifier'])
        session.add(binding)


def delete_vlan_binding(context, record_dict):
    """Delete vlan bindings of a given physical port."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        if (record_dict['vlan'] is not None and
                record_dict['logical_switch_uuid']):
            session.query(models.VlanBindings).filter_by(
                port_uuid=record_dict['port_uuid'], vlan=record_dict['vlan'],
                logical_switch_uuid=record_dict['logical_switch_uuid'],
                ovsdb_identifier=record_dict['ovsdb_identifier']).delete()


def add_physical_locator(context, record_dict):
    """Insert a new physical locator."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        locator = models.PhysicalLocators(
            uuid=record_dict['uuid'],
            dst_ip=record_dict['dst_ip'],
            ovsdb_identifier=record_dict['ovsdb_identifier'])
        session.add(locator)


def delete_physical_locator(context, record_dict):
    """Delete physical locator that matches the supplied uuid."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        if record_dict['uuid']:
            session.query(models.PhysicalLocators).filter_by(
                uuid=record_dict['uuid'],
                ovsdb_identifier=record_dict['ovsdb_identifier']).delete()


def add_physical_switch(context, record_dict):
    """Insert a new physical switch."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        physical_switch = models.PhysicalSwitches(
            uuid=record_dict['uuid'],
            name=record_dict['name'],
            tunnel_ip=record_dict['tunnel_ip'],
            ovsdb_identifier=record_dict['ovsdb_identifier'],
            switch_fault_status=record_dict['switch_fault_status'])
        session.add(physical_switch)


def delete_physical_switch(context, record_dict):
    """Delete physical switch that matches the supplied uuid."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        if record_dict['uuid']:
            session.query(models.PhysicalSwitches).filter_by(
                uuid=record_dict['uuid'],
                ovsdb_identifier=record_dict['ovsdb_identifier']).delete()


def add_logical_switch(context, record_dict):
    """Insert a new logical switch."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        logical_switch = models.LogicalSwitches(
            uuid=record_dict['uuid'],
            name=record_dict['name'],
            key=record_dict['key'],
            ovsdb_identifier=record_dict['ovsdb_identifier'])
        session.add(logical_switch)


def delete_logical_switch(context, record_dict):
    """delete logical switch that matches the supplied uuid."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        if record_dict['uuid']:
            session.query(models.LogicalSwitches).filter_by(
                uuid=record_dict['uuid'],
                ovsdb_identifier=record_dict['ovsdb_identifier']).delete()


def add_physical_port(context, record_dict):
    """Insert a new physical port."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        physical_port = models.PhysicalPorts(
            uuid=record_dict['uuid'],
            name=record_dict['name'],
            physical_switch_id=record_dict['physical_switch_id'],
            ovsdb_identifier=record_dict['ovsdb_identifier'],
            port_fault_status=record_dict['port_fault_status'])
        session.add(physical_port)


def update_physical_ports_status(context, record_dict):
    """Update physical port fault status."""
    with db_api.CONTEXT_WRITER.using(context):
        (context.session.query(models.PhysicalPorts).
         filter(models.PhysicalPorts.uuid == record_dict['uuid']).
         update({'port_fault_status': record_dict['port_fault_status']},
         synchronize_session=False))


def update_physical_switch_status(context, record_dict):
    """Update physical switch fault status."""
    with context.session.begin():
        (context.session.query(models.PhysicalSwitches).
         filter(models.PhysicalSwitches.uuid == record_dict['uuid']).
         update({'switch_fault_status': record_dict['switch_fault_status']},
         synchronize_session=False))


def delete_physical_port(context, record_dict):
    """Delete physical port that matches the supplied uuid."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        if record_dict['uuid']:
            session.query(models.PhysicalPorts).filter_by(
                uuid=record_dict['uuid'],
                ovsdb_identifier=record_dict['ovsdb_identifier']).delete()


def add_ucast_mac_local(context, record_dict):
    """Insert a new ucast mac local."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        ucast_mac_local = models.UcastMacsLocals(
            uuid=record_dict['uuid'],
            mac=record_dict['mac'],
            logical_switch_id=record_dict['logical_switch_id'],
            physical_locator_id=record_dict['physical_locator_id'],
            ip_address=record_dict['ip_address'],
            ovsdb_identifier=record_dict['ovsdb_identifier'])
        session.add(ucast_mac_local)


def delete_ucast_mac_local(context, record_dict):
    """Delete ucast mac local that matches the supplied uuid."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        if record_dict['uuid']:
            session.query(models.UcastMacsLocals).filter_by(
                uuid=record_dict['uuid'],
                ovsdb_identifier=record_dict['ovsdb_identifier']).delete()


def add_ucast_mac_remote(context, record_dict):
    """Insert a new ucast mac remote."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        ucast_mac_remote = models.UcastMacsRemotes(
            uuid=record_dict['uuid'],
            mac=record_dict['mac'],
            logical_switch_id=record_dict['logical_switch_id'],
            physical_locator_id=record_dict['physical_locator_id'],
            ip_address=record_dict['ip_address'],
            ovsdb_identifier=record_dict['ovsdb_identifier'])
        session.add(ucast_mac_remote)


def update_ucast_mac_remote(context, rec_dict):
    """Update ucast mac remote."""
    try:
        with db_api.CONTEXT_WRITER.using(context):
            (context.session.query(models.UcastMacsRemotes).filter_by(
                uuid=rec_dict['uuid'],
                ovsdb_identifier=rec_dict['ovsdb_identifier']).update(
                    {'physical_locator_id': rec_dict['physical_locator_id'],
                     'ip_address': rec_dict['ip_address']},
                    synchronize_session=False))
    except exc.NoResultFound:
        LOG.debug('no Remote mac found for %s and %s',
                  rec_dict['uuid'],
                  rec_dict['ovsdb_identifier'])


def delete_ucast_mac_remote(context, record_dict):
    """Delete ucast mac remote that matches the supplied uuid."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        if record_dict['uuid']:
            session.query(models.UcastMacsRemotes).filter_by(
                uuid=record_dict['uuid'],
                ovsdb_identifier=record_dict['ovsdb_identifier']).delete()


def get_physical_port(context, record_dict):
    """Get physical port that matches the uuid and ovsdb_identifier."""
    try:
        query = context.session.query(models.PhysicalPorts)
        physical_port = query.filter_by(
            uuid=record_dict['uuid'],
            ovsdb_identifier=record_dict['ovsdb_identifier']).one()
    except exc.NoResultFound:
        LOG.debug('no physical port found for %s and %s',
                  record_dict['uuid'],
                  record_dict['ovsdb_identifier'])
        return
    return physical_port


def get_logical_switch(context, record_dict):
    """Get logical switch that matches the uuid and ovsdb_identifier."""
    try:
        query = context.session.query(models.LogicalSwitches)
        logical_switch = query.filter_by(
            uuid=record_dict['uuid'],
            ovsdb_identifier=record_dict['ovsdb_identifier']).one()
    except exc.NoResultFound:
        LOG.debug('no logical switch found for %s and %s',
                  record_dict['uuid'],
                  record_dict['ovsdb_identifier'])
        return
    return logical_switch


def get_all_logical_switches_by_name(context, name):
    """Get logical switch that matches the supplied name."""
    query = context.session.query(models.LogicalSwitches)
    return query.filter_by(name=name).all()


def get_ucast_mac_remote(context, record_dict):
    """Get ucast macs remote that matches the uuid and ovsdb_identifier."""
    try:
        query = context.session.query(models.UcastMacsRemotes)
        remote_mac = query.filter_by(
            uuid=record_dict['uuid'],
            ovsdb_identifier=record_dict['ovsdb_identifier']).one()
    except exc.NoResultFound:
        LOG.debug('no Remote mac found for %s and %s',
                  record_dict['uuid'],
                  record_dict['ovsdb_identifier'])
        return
    return remote_mac


def get_ucast_mac_local(context, record_dict):
    """Get ucast macs local that matches the uuid and ovsdb_identifier."""
    try:
        with db_api.CONTEXT_READER.using(context):
            query = context.session.query(models.UcastMacsLocals)
            local_mac = query.filter_by(
                uuid=record_dict['uuid'],
                ovsdb_identifier=record_dict['ovsdb_identifier']).one()
    except exc.NoResultFound:
        LOG.debug('no Local mac found for %s and %s',
                  record_dict['uuid'],
                  record_dict['ovsdb_identifier'])
        return
    return local_mac


def get_ucast_mac_remote_by_mac_and_ls(context, record_dict):
    """Get ucast macs remote that matches the MAC address and

       ovsdb_identifier.
    """
    try:
        query = context.session.query(models.UcastMacsRemotes)
        remote_mac = query.filter_by(
            mac=record_dict['mac'],
            ovsdb_identifier=record_dict['ovsdb_identifier'],
            logical_switch_id=record_dict['logical_switch_uuid']).one()
    except exc.NoResultFound:
        LOG.debug('no Remote mac found for %s and %s',
                  record_dict['mac'],
                  record_dict['logical_switch_uuid'])
        return
    return remote_mac


def get_physical_switch(context, record_dict):
    """Get physical switch that matches the uuid and ovsdb_identifier."""
    try:
        query = context.session.query(models.PhysicalSwitches)
        physical_switch = query.filter_by(
            uuid=record_dict['uuid'],
            ovsdb_identifier=record_dict['ovsdb_identifier']).one()
    except exc.NoResultFound:
        LOG.debug('no physical switch found for %s and %s',
                  record_dict['uuid'],
                  record_dict['ovsdb_identifier'])
        return
    return physical_switch


def get_physical_locator(context, record_dict):
    """Get physical locator that matches the supplied uuid."""
    try:
        query = context.session.query(models.PhysicalLocators)
        physical_locator = query.filter_by(
            uuid=record_dict['uuid'],
            ovsdb_identifier=record_dict['ovsdb_identifier']).one()
    except exc.NoResultFound:
        LOG.debug('no physical locator found for %s and %s',
                  record_dict['uuid'],
                  record_dict['ovsdb_identifier'])
        return
    return physical_locator


def get_physical_locator_by_dst_ip(context, record_dict):
    """Get physical locator that matches the supplied destination IP."""
    try:
        query = context.session.query(models.PhysicalLocators)
        physical_locator = query.filter_by(
            dst_ip=record_dict['dst_ip'],
            ovsdb_identifier=record_dict['ovsdb_identifier']).one()
    except exc.NoResultFound:
        LOG.debug('no physical locator found for %s and %s',
                  record_dict['dst_ip'],
                  record_dict['ovsdb_identifier'])
        return
    return physical_locator


def get_logical_switch_by_name(context, record_dict):
    """Get logical switch that matches the supplied name."""
    try:
        with db_api.CONTEXT_READER.using(context):
            query = context.session.query(models.LogicalSwitches)
            logical_switch = query.filter_by(
                name=record_dict['logical_switch_name'],
                ovsdb_identifier=record_dict['ovsdb_identifier']).one()
    except exc.NoResultFound:
        LOG.debug('no logical switch found for %s and %s',
                  record_dict['logical_switch_name'],
                  record_dict['ovsdb_identifier'])
        return
    return logical_switch


def get_all_vlan_bindings_by_physical_port(context, record_dict):
    """Get vlan bindings that matches the supplied physical port."""
    query = context.session.query(models.VlanBindings)
    return query.filter_by(
        port_uuid=record_dict['uuid'],
        ovsdb_identifier=record_dict['ovsdb_identifier']).all()


def get_vlan_binding(context, record_dict):
    """Get vlan bindings that matches the supplied physical port."""
    try:
        query = context.session.query(models.VlanBindings)
        vlan_binding = query.filter_by(
            port_uuid=record_dict['port_uuid'],
            vlan=record_dict['vlan'],
            logical_switch_uuid=record_dict['logical_switch_uuid'],
            ovsdb_identifier=record_dict['ovsdb_identifier']).one()
    except exc.NoResultFound:
        LOG.debug('no vlan binding found for %s and %s',
                  record_dict['port_uuid'],
                  record_dict['ovsdb_identifier'])
        return
    return vlan_binding


def get_physical_switch_by_name(context, name):
    """Get logical switch that matches the supplied name."""
    query = context.session.query(models.PhysicalSwitches)
    return query.filter_by(name=name).first()


def get_physical_port_by_name_and_ps(context, record_dict):
    """Get vlan bindings that matches the supplied physical port."""
    try:
        query = context.session.query(models.PhysicalPorts)
        physical_port = query.filter_by(
            name=record_dict['interface_name'],
            physical_switch_id=record_dict['physical_switch_id'],
            ovsdb_identifier=record_dict['ovsdb_identifier']).one()
    except exc.NoResultFound:
        LOG.debug('no physical port found for %s and %s',
                  record_dict['physical_switch_id'],
                  record_dict['interface_name'])
        return
    return physical_port


def get_all_physical_switches_by_ovsdb_id(context, ovsdb_identifier):
    """Get Physical Switches that match the supplied ovsdb identifier."""
    query = context.session.query(models.PhysicalSwitches)
    return query.filter_by(
        ovsdb_identifier=ovsdb_identifier).all()


def get_all_logical_switches_by_ovsdb_id(context, ovsdb_identifier):
    """Get logical Switches that match the supplied ovsdb identifier."""
    query = context.session.query(models.LogicalSwitches)
    return query.filter_by(
        ovsdb_identifier=ovsdb_identifier).all()


def get_all_vlan_bindings_by_logical_switch(context, record_dict):
    """Get Vlan bindings that match the supplied logical switch."""
    query = context.session.query(models.VlanBindings)
    return query.filter_by(
        logical_switch_uuid=record_dict['logical_switch_id'],
        ovsdb_identifier=record_dict['ovsdb_identifier']).all()


def add_pending_ucast_mac_remote(context, operation,
                                 ovsdb_identifier,
                                 logical_switch_id,
                                 physical_locator,
                                 mac_remotes):
    """Insert a pending ucast_mac_remote (insert/update/delete)."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        for mac in mac_remotes:
            pending_mac = models.PendingUcastMacsRemote(
                uuid=mac.get('uuid', None),
                mac=mac['mac'],
                logical_switch_uuid=logical_switch_id,
                vm_ip=mac['ip_address'],
                ovsdb_identifier=ovsdb_identifier,
                operation=operation,
                timestamp=timeutils.utcnow())
            if physical_locator:
                pending_mac['dst_ip'] = physical_locator.get('dst_ip', None)
                pending_mac['locator_uuid'] = physical_locator.get('uuid',
                                                                   None)
            session.add(pending_mac)


def delete_pending_ucast_mac_remote(context, operation,
                                    ovsdb_identifier,
                                    logical_switch_id,
                                    mac_remote):
    """Delete a pending ucast_mac_remote."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        if (mac_remote and logical_switch_id and
                ovsdb_identifier and operation):
            query = session.query(models.PendingUcastMacsRemote).filter_by(
                mac=mac_remote,
                ovsdb_identifier=ovsdb_identifier,
                logical_switch_uuid=logical_switch_id,
                operation=operation)
            row_count = query.count()
            query.delete()
            return row_count


def get_pending_ucast_mac_remote(context, ovsdb_identifier, mac,
                                 logical_switch_uuid):
    """Get pending mac that matches the supplied parameters."""
    try:
        query = context.session.query(models.PendingUcastMacsRemote)
        pending_mac = query.filter_by(
            ovsdb_identifier=ovsdb_identifier,
            logical_switch_uuid=logical_switch_uuid,
            mac=mac).one()
        return pending_mac
    except exc.NoResultFound:
        return


def get_all_pending_remote_macs_in_asc_order(context, ovsdb_identifier):
    """Get all the pending remote macs in ascending order of timestamp."""
    with db_api.CONTEXT_READER.using(context):
        session = context.session
        return session.query(
            models.PendingUcastMacsRemote).filter_by(
            ovsdb_identifier=ovsdb_identifier).order_by(
            asc(models.PendingUcastMacsRemote.timestamp)).all()


def get_all_ucast_mac_remote_by_ls(context, record_dict):
    """Get ucast macs remote that matches ls_id and ovsdb_identifier."""
    with db_api.CONTEXT_READER.using(context):
        session = context.session
        return session.query(models.UcastMacsRemotes).filter_by(
            ovsdb_identifier=record_dict['ovsdb_identifier'],
            logical_switch_id=record_dict['logical_switch_id']).all()


def delete_all_physical_locators_by_ovsdb_identifier(context,
                                                     ovsdb_identifier):
    """Delete all physical locators based on ovsdb identifier."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        session.query(models.PhysicalLocators).filter_by(
            ovsdb_identifier=ovsdb_identifier).delete()


def delete_all_physical_switches_by_ovsdb_identifier(context,
                                                     ovsdb_identifier):
    """Delete all physical switches based on ovsdb identifier."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        session.query(models.PhysicalSwitches).filter_by(
            ovsdb_identifier=ovsdb_identifier).delete()


def delete_all_physical_ports_by_ovsdb_identifier(context,
                                                  ovsdb_identifier):
    """Delete all physical ports based on ovsdb identifier."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        session.query(models.PhysicalPorts).filter_by(
            ovsdb_identifier=ovsdb_identifier).delete()


def delete_all_logical_switches_by_ovsdb_identifier(context,
                                                    ovsdb_identifier):
    """Delete all physical switches based on ovsdb identifier."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        session.query(models.LogicalSwitches).filter_by(
            ovsdb_identifier=ovsdb_identifier).delete()


def delete_all_ucast_macs_locals_by_ovsdb_identifier(context,
                                                     ovsdb_identifier):
    """Delete all ucast mac locals based on ovsdb identifier."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        session.query(models.UcastMacsLocals).filter_by(
            ovsdb_identifier=ovsdb_identifier).delete()


def delete_all_ucast_macs_remotes_by_ovsdb_identifier(context,
                                                      ovsdb_identifier):
    """Delete all ucast mac remotes based on ovsdb identifier."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        session.query(models.UcastMacsRemotes).filter_by(
            ovsdb_identifier=ovsdb_identifier).delete()


def delete_all_vlan_bindings_by_ovsdb_identifier(context,
                                                 ovsdb_identifier):
    """Delete all vlan bindings based on ovsdb identifier."""
    session = context.session
    with db_api.CONTEXT_WRITER.using(context):
        session.query(models.VlanBindings).filter_by(
            ovsdb_identifier=ovsdb_identifier).delete()



# Copyright (c) 2015 Hewlett-Packard Development Company, L.P.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import copy
import datetime
from unittest import mock

from oslo_config import cfg
from oslo_service import loopingcall
from oslo_utils import timeutils

from neutron.agent.common import utils
from neutron.db import agents_db
from neutron import manager
from neutron.plugins.ml2 import rpc
from neutron.tests import base
from neutron_lib.agent import topics
from neutron_lib import context as neutron_context

from networking_l2gw.services.l2gateway import agent_scheduler
from networking_l2gw.services.l2gateway.common import config as l2gw_config
from networking_l2gw.services.l2gateway.common import constants as srv_const
from networking_l2gw.services.l2gateway.common import topics as l2gw_topics
from networking_l2gw.services.l2gateway.service_drivers import agent_api


def make_active_agent(fake_id, fake_agent_type, config=None):
    agent_dict = dict(id=fake_id,
                      agent_type=fake_agent_type,
                      host='localhost_' + str(fake_id),
                      heartbeat_timestamp=timeutils.utcnow(),
                      started_at=timeutils.utcnow(),
                      configurations=config)
    return agent_dict


def make_inactive_agent(fake_id, fake_agent_type, delta, config=None):
    agent_dict = dict(id=fake_id,
                      agent_type=fake_agent_type,
                      host='remotehost_' + str(fake_id),
                      heartbeat_timestamp=(timeutils.utcnow() - datetime.
                                           timedelta(delta)),
                      configurations=config)
    return agent_dict


class FakePlugin(agents_db.AgentDbMixin):

    def __init__(self):
        self.notifier = rpc.AgentNotifierApi(topics.AGENT)


class TestAgentScheduler(base.BaseTestCase):

    fake_a_agent_list = []
    fake_i_agent_list = []

    def setUp(self):
        super(TestAgentScheduler, self).setUp()
        l2gw_config.register_l2gw_opts_helper()
        cfg.CONF.set_override('core_plugin',
                              "neutron.plugins.ml2.plugin.Ml2Plugin")
        self.plugin = FakePlugin()
        self.agent_rpc = agent_api.L2gatewayAgentApi(
            l2gw_topics.L2GATEWAY_AGENT, cfg.CONF.host)
        self.context = neutron_context.get_admin_context()
        cfg.CONF.set_override('agent_down_time', 10)
        cfg.CONF.set_override('periodic_monitoring_interval', 5)
        self.agentsch = agent_scheduler.L2GatewayAgentScheduler(self.agent_rpc,
                                                                cfg.CONF)
        self.agentsch._plugin = self.plugin
        self.agentsch.context = self.context
        self.agentsch.agent_ext_support = True
        self.LOG = agent_scheduler.LOG

    def populate_agent_lists(self, config=None):
        self.fake_a_agent_list = []
        self.fake_a_agent_list.append(make_active_agent(
            '1000', srv_const.AGENT_TYPE_L2GATEWAY, config))

        self.fake_i_agent_list = []
        self.fake_i_agent_list.append(make_inactive_agent(
            '2000', srv_const.AGENT_TYPE_L2GATEWAY, 52, config))

    @mock.patch.object(loopingcall, 'FixedIntervalLoopingCall')
    @mock.patch.object(agent_scheduler.LOG, 'debug')
    @mock.patch.object(agent_scheduler.LOG, 'error')
    def test_initialize_thread(self, err, debug, loop_call):
        self.agentsch.initialize_thread()
        self.assertTrue(loop_call.called)
        self.assertTrue(debug.called)
        self.assertFalse(err.called)

    @mock.patch.object(loopingcall, 'FixedIntervalLoopingCall',
                       side_effect=RuntimeError)
    def test_initialize_thread_loop_call_exception(self, loop_call):
        with mock.patch.object(self.LOG, 'error') as log_err:
            self.agentsch.initialize_thread()
            self.assertTrue(loop_call.called)
            self.assertTrue(log_err.called)

    @mock.patch.object(manager, 'NeutronManager')
    def test_select_agent_type_one_active(self, mgr):
        config = {srv_const.L2GW_AGENT_TYPE: ''}
        self.populate_agent_lists(config)

        with mock.patch.object(self.LOG, 'exception'):
            self.agentsch._l2gwplugin = mock.Mock()
            self.agentsch._select_agent_type(self.context,
                                             self.fake_a_agent_list)
            self.agentsch.l2gwplugin.agent_rpc.set_monitor_agent_called_with(
                self.context, self.fake_a_agent_list[0]['host'])

    @mock.patch.object(manager, 'NeutronManager')
    def test_select_agent_type_multiple_active(self, mgr):
        config = {srv_const.L2GW_AGENT_TYPE: ''}
        self.populate_agent_lists(config)
        self.fake_a_agent_list.append(make_active_agent(
            '1001', srv_const.AGENT_TYPE_L2GATEWAY, config))
        self.agentsch._l2gwplugin = mock.Mock()

        with mock.patch.object(self.LOG, 'exception'):
            self.agentsch._select_agent_type(self.context,
                                             self.fake_a_agent_list)
            self.agentsch.l2gwplugin.agent_rpc.set_monitor_agent_called_with(
                self.context, self.fake_a_agent_list[0]['host'])

    def test_monitor_agent_state(self):
        config = {srv_const.L2GW_AGENT_TYPE: ''}
        self.populate_agent_lists(config)
        fake_all_agent_list = copy.deepcopy(self.fake_i_agent_list)
        fake_all_agent_list.extend(self.fake_a_agent_list)
        self.fake_a_agent_list.append(make_active_agent(
            '1001', srv_const.AGENT_TYPE_L2GATEWAY, config))

        with mock.patch.object(self.agentsch,
                               '_select_agent_type') as select_agent, \
                mock.patch.object(
                    self.plugin, 'get_agents',
                    return_value=fake_all_agent_list) as get_agent_list, \
                mock.patch.object(utils, 'is_agent_down',
                                  return_value=False) as is_agt:
            self.agentsch.monitor_agent_state()
            self.assertTrue(get_agent_list.called)
            self.assertTrue(select_agent.called)
            self.assertTrue(is_agt.called)

    def test_monitor_agent_state_exception_get_agents(self):
        with mock.patch.object(
                self.plugin, 'get_agents',
                side_effect=Exception) as get_agent_list, \
                mock.patch.object(self.LOG, 'exception') as exception_log:
            self.agentsch.monitor_agent_state()
            self.assertTrue(get_agent_list.called)
            self.assertTrue(exception_log.called)
